from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...extensions import UnknownType
from ...models.aig_generate_input_async_task import AIGGenerateInputAsyncTask
from ...models.aop_process_output_async_task import AOPProcessOutputAsyncTask
from ...models.async_task import AsyncTask
from ...models.autofill_parts_async_task import AutofillPartsAsyncTask
from ...models.autofill_translations_async_task import AutofillTranslationsAsyncTask
from ...models.bulk_create_aa_sequences_async_task import BulkCreateAaSequencesAsyncTask
from ...models.bulk_create_containers_async_task import BulkCreateContainersAsyncTask
from ...models.bulk_create_custom_entities_async_task import BulkCreateCustomEntitiesAsyncTask
from ...models.bulk_create_dna_oligos_async_task import BulkCreateDnaOligosAsyncTask
from ...models.bulk_create_dna_sequences_async_task import BulkCreateDnaSequencesAsyncTask
from ...models.bulk_create_features_async_task import BulkCreateFeaturesAsyncTask
from ...models.bulk_create_rna_oligos_async_task import BulkCreateRnaOligosAsyncTask
from ...models.bulk_create_rna_sequences_async_task import BulkCreateRnaSequencesAsyncTask
from ...models.bulk_register_entities_async_task import BulkRegisterEntitiesAsyncTask
from ...models.bulk_update_aa_sequences_async_task import BulkUpdateAaSequencesAsyncTask
from ...models.bulk_update_containers_async_task import BulkUpdateContainersAsyncTask
from ...models.bulk_update_custom_entities_async_task import BulkUpdateCustomEntitiesAsyncTask
from ...models.bulk_update_dna_oligos_async_task import BulkUpdateDnaOligosAsyncTask
from ...models.bulk_update_dna_sequences_async_task import BulkUpdateDnaSequencesAsyncTask
from ...models.bulk_update_rna_oligos_async_task import BulkUpdateRnaOligosAsyncTask
from ...models.bulk_update_rna_sequences_async_task import BulkUpdateRnaSequencesAsyncTask
from ...models.create_consensus_alignment_async_task import CreateConsensusAlignmentAsyncTask
from ...models.create_nucleotide_consensus_alignment_async_task import (
    CreateNucleotideConsensusAlignmentAsyncTask,
)
from ...models.create_nucleotide_template_alignment_async_task import (
    CreateNucleotideTemplateAlignmentAsyncTask,
)
from ...models.create_template_alignment_async_task import CreateTemplateAlignmentAsyncTask
from ...models.exports_async_task import ExportsAsyncTask
from ...models.find_matching_regions_async_task import FindMatchingRegionsAsyncTask
from ...models.find_matching_regions_dna_async_task import FindMatchingRegionsDnaAsyncTask
from ...models.not_found_error import NotFoundError
from ...models.transfers_async_task import TransfersAsyncTask
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    task_id: str,
) -> Dict[str, Any]:
    url = "{}/tasks/{task_id}".format(client.base_url, task_id=task_id)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        Union[
            AsyncTask,
            CreateTemplateAlignmentAsyncTask,
            CreateConsensusAlignmentAsyncTask,
            CreateNucleotideTemplateAlignmentAsyncTask,
            CreateNucleotideConsensusAlignmentAsyncTask,
            BulkCreateDnaSequencesAsyncTask,
            BulkUpdateDnaSequencesAsyncTask,
            BulkCreateRnaSequencesAsyncTask,
            BulkUpdateRnaSequencesAsyncTask,
            AutofillPartsAsyncTask,
            AutofillTranslationsAsyncTask,
            BulkRegisterEntitiesAsyncTask,
            BulkCreateDnaOligosAsyncTask,
            BulkUpdateDnaOligosAsyncTask,
            BulkCreateRnaOligosAsyncTask,
            BulkCreateAaSequencesAsyncTask,
            BulkCreateCustomEntitiesAsyncTask,
            BulkUpdateCustomEntitiesAsyncTask,
            BulkCreateContainersAsyncTask,
            BulkUpdateContainersAsyncTask,
            BulkUpdateAaSequencesAsyncTask,
            BulkUpdateRnaOligosAsyncTask,
            TransfersAsyncTask,
            AOPProcessOutputAsyncTask,
            AIGGenerateInputAsyncTask,
            ExportsAsyncTask,
            BulkCreateFeaturesAsyncTask,
            FindMatchingRegionsAsyncTask,
            FindMatchingRegionsDnaAsyncTask,
            UnknownType,
        ],
        NotFoundError,
    ]
]:
    if response.status_code == 200:

        def _parse_response_200(
            data: Union[Dict[str, Any]]
        ) -> Union[
            AsyncTask,
            CreateTemplateAlignmentAsyncTask,
            CreateConsensusAlignmentAsyncTask,
            CreateNucleotideTemplateAlignmentAsyncTask,
            CreateNucleotideConsensusAlignmentAsyncTask,
            BulkCreateDnaSequencesAsyncTask,
            BulkUpdateDnaSequencesAsyncTask,
            BulkCreateRnaSequencesAsyncTask,
            BulkUpdateRnaSequencesAsyncTask,
            AutofillPartsAsyncTask,
            AutofillTranslationsAsyncTask,
            BulkRegisterEntitiesAsyncTask,
            BulkCreateDnaOligosAsyncTask,
            BulkUpdateDnaOligosAsyncTask,
            BulkCreateRnaOligosAsyncTask,
            BulkCreateAaSequencesAsyncTask,
            BulkCreateCustomEntitiesAsyncTask,
            BulkUpdateCustomEntitiesAsyncTask,
            BulkCreateContainersAsyncTask,
            BulkUpdateContainersAsyncTask,
            BulkUpdateAaSequencesAsyncTask,
            BulkUpdateRnaOligosAsyncTask,
            TransfersAsyncTask,
            AOPProcessOutputAsyncTask,
            AIGGenerateInputAsyncTask,
            ExportsAsyncTask,
            BulkCreateFeaturesAsyncTask,
            FindMatchingRegionsAsyncTask,
            FindMatchingRegionsDnaAsyncTask,
            UnknownType,
        ]:
            response_200: Union[
                AsyncTask,
                CreateTemplateAlignmentAsyncTask,
                CreateConsensusAlignmentAsyncTask,
                CreateNucleotideTemplateAlignmentAsyncTask,
                CreateNucleotideConsensusAlignmentAsyncTask,
                BulkCreateDnaSequencesAsyncTask,
                BulkUpdateDnaSequencesAsyncTask,
                BulkCreateRnaSequencesAsyncTask,
                BulkUpdateRnaSequencesAsyncTask,
                AutofillPartsAsyncTask,
                AutofillTranslationsAsyncTask,
                BulkRegisterEntitiesAsyncTask,
                BulkCreateDnaOligosAsyncTask,
                BulkUpdateDnaOligosAsyncTask,
                BulkCreateRnaOligosAsyncTask,
                BulkCreateAaSequencesAsyncTask,
                BulkCreateCustomEntitiesAsyncTask,
                BulkUpdateCustomEntitiesAsyncTask,
                BulkCreateContainersAsyncTask,
                BulkUpdateContainersAsyncTask,
                BulkUpdateAaSequencesAsyncTask,
                BulkUpdateRnaOligosAsyncTask,
                TransfersAsyncTask,
                AOPProcessOutputAsyncTask,
                AIGGenerateInputAsyncTask,
                ExportsAsyncTask,
                BulkCreateFeaturesAsyncTask,
                FindMatchingRegionsAsyncTask,
                FindMatchingRegionsDnaAsyncTask,
                UnknownType,
            ]
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = AsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = CreateTemplateAlignmentAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = CreateConsensusAlignmentAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = CreateNucleotideTemplateAlignmentAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = CreateNucleotideConsensusAlignmentAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkCreateDnaSequencesAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkUpdateDnaSequencesAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkCreateRnaSequencesAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkUpdateRnaSequencesAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = AutofillPartsAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = AutofillTranslationsAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkRegisterEntitiesAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkCreateDnaOligosAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkUpdateDnaOligosAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkCreateRnaOligosAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkCreateAaSequencesAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkCreateCustomEntitiesAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkUpdateCustomEntitiesAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkCreateContainersAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkUpdateContainersAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkUpdateAaSequencesAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkUpdateRnaOligosAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = TransfersAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = AOPProcessOutputAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = AIGGenerateInputAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = ExportsAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = BulkCreateFeaturesAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = FindMatchingRegionsAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200 = FindMatchingRegionsDnaAsyncTask.from_dict(data, strict=True)

                return response_200
            except:  # noqa: E722
                pass
            return UnknownType(data)

        response_200 = _parse_response_200(response.json())

        return response_200
    if response.status_code == 404:
        response_404 = NotFoundError.from_dict(response.json(), strict=False)

        return response_404
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        Union[
            AsyncTask,
            CreateTemplateAlignmentAsyncTask,
            CreateConsensusAlignmentAsyncTask,
            CreateNucleotideTemplateAlignmentAsyncTask,
            CreateNucleotideConsensusAlignmentAsyncTask,
            BulkCreateDnaSequencesAsyncTask,
            BulkUpdateDnaSequencesAsyncTask,
            BulkCreateRnaSequencesAsyncTask,
            BulkUpdateRnaSequencesAsyncTask,
            AutofillPartsAsyncTask,
            AutofillTranslationsAsyncTask,
            BulkRegisterEntitiesAsyncTask,
            BulkCreateDnaOligosAsyncTask,
            BulkUpdateDnaOligosAsyncTask,
            BulkCreateRnaOligosAsyncTask,
            BulkCreateAaSequencesAsyncTask,
            BulkCreateCustomEntitiesAsyncTask,
            BulkUpdateCustomEntitiesAsyncTask,
            BulkCreateContainersAsyncTask,
            BulkUpdateContainersAsyncTask,
            BulkUpdateAaSequencesAsyncTask,
            BulkUpdateRnaOligosAsyncTask,
            TransfersAsyncTask,
            AOPProcessOutputAsyncTask,
            AIGGenerateInputAsyncTask,
            ExportsAsyncTask,
            BulkCreateFeaturesAsyncTask,
            FindMatchingRegionsAsyncTask,
            FindMatchingRegionsDnaAsyncTask,
            UnknownType,
        ],
        NotFoundError,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    task_id: str,
) -> Response[
    Union[
        Union[
            AsyncTask,
            CreateTemplateAlignmentAsyncTask,
            CreateConsensusAlignmentAsyncTask,
            CreateNucleotideTemplateAlignmentAsyncTask,
            CreateNucleotideConsensusAlignmentAsyncTask,
            BulkCreateDnaSequencesAsyncTask,
            BulkUpdateDnaSequencesAsyncTask,
            BulkCreateRnaSequencesAsyncTask,
            BulkUpdateRnaSequencesAsyncTask,
            AutofillPartsAsyncTask,
            AutofillTranslationsAsyncTask,
            BulkRegisterEntitiesAsyncTask,
            BulkCreateDnaOligosAsyncTask,
            BulkUpdateDnaOligosAsyncTask,
            BulkCreateRnaOligosAsyncTask,
            BulkCreateAaSequencesAsyncTask,
            BulkCreateCustomEntitiesAsyncTask,
            BulkUpdateCustomEntitiesAsyncTask,
            BulkCreateContainersAsyncTask,
            BulkUpdateContainersAsyncTask,
            BulkUpdateAaSequencesAsyncTask,
            BulkUpdateRnaOligosAsyncTask,
            TransfersAsyncTask,
            AOPProcessOutputAsyncTask,
            AIGGenerateInputAsyncTask,
            ExportsAsyncTask,
            BulkCreateFeaturesAsyncTask,
            FindMatchingRegionsAsyncTask,
            FindMatchingRegionsDnaAsyncTask,
            UnknownType,
        ],
        NotFoundError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        task_id=task_id,
    )

    response = client.httpx_client.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    task_id: str,
) -> Optional[
    Union[
        Union[
            AsyncTask,
            CreateTemplateAlignmentAsyncTask,
            CreateConsensusAlignmentAsyncTask,
            CreateNucleotideTemplateAlignmentAsyncTask,
            CreateNucleotideConsensusAlignmentAsyncTask,
            BulkCreateDnaSequencesAsyncTask,
            BulkUpdateDnaSequencesAsyncTask,
            BulkCreateRnaSequencesAsyncTask,
            BulkUpdateRnaSequencesAsyncTask,
            AutofillPartsAsyncTask,
            AutofillTranslationsAsyncTask,
            BulkRegisterEntitiesAsyncTask,
            BulkCreateDnaOligosAsyncTask,
            BulkUpdateDnaOligosAsyncTask,
            BulkCreateRnaOligosAsyncTask,
            BulkCreateAaSequencesAsyncTask,
            BulkCreateCustomEntitiesAsyncTask,
            BulkUpdateCustomEntitiesAsyncTask,
            BulkCreateContainersAsyncTask,
            BulkUpdateContainersAsyncTask,
            BulkUpdateAaSequencesAsyncTask,
            BulkUpdateRnaOligosAsyncTask,
            TransfersAsyncTask,
            AOPProcessOutputAsyncTask,
            AIGGenerateInputAsyncTask,
            ExportsAsyncTask,
            BulkCreateFeaturesAsyncTask,
            FindMatchingRegionsAsyncTask,
            FindMatchingRegionsDnaAsyncTask,
            UnknownType,
        ],
        NotFoundError,
    ]
]:
    """ Get a task by id """

    return sync_detailed(
        client=client,
        task_id=task_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    task_id: str,
) -> Response[
    Union[
        Union[
            AsyncTask,
            CreateTemplateAlignmentAsyncTask,
            CreateConsensusAlignmentAsyncTask,
            CreateNucleotideTemplateAlignmentAsyncTask,
            CreateNucleotideConsensusAlignmentAsyncTask,
            BulkCreateDnaSequencesAsyncTask,
            BulkUpdateDnaSequencesAsyncTask,
            BulkCreateRnaSequencesAsyncTask,
            BulkUpdateRnaSequencesAsyncTask,
            AutofillPartsAsyncTask,
            AutofillTranslationsAsyncTask,
            BulkRegisterEntitiesAsyncTask,
            BulkCreateDnaOligosAsyncTask,
            BulkUpdateDnaOligosAsyncTask,
            BulkCreateRnaOligosAsyncTask,
            BulkCreateAaSequencesAsyncTask,
            BulkCreateCustomEntitiesAsyncTask,
            BulkUpdateCustomEntitiesAsyncTask,
            BulkCreateContainersAsyncTask,
            BulkUpdateContainersAsyncTask,
            BulkUpdateAaSequencesAsyncTask,
            BulkUpdateRnaOligosAsyncTask,
            TransfersAsyncTask,
            AOPProcessOutputAsyncTask,
            AIGGenerateInputAsyncTask,
            ExportsAsyncTask,
            BulkCreateFeaturesAsyncTask,
            FindMatchingRegionsAsyncTask,
            FindMatchingRegionsDnaAsyncTask,
            UnknownType,
        ],
        NotFoundError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        task_id=task_id,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    task_id: str,
) -> Optional[
    Union[
        Union[
            AsyncTask,
            CreateTemplateAlignmentAsyncTask,
            CreateConsensusAlignmentAsyncTask,
            CreateNucleotideTemplateAlignmentAsyncTask,
            CreateNucleotideConsensusAlignmentAsyncTask,
            BulkCreateDnaSequencesAsyncTask,
            BulkUpdateDnaSequencesAsyncTask,
            BulkCreateRnaSequencesAsyncTask,
            BulkUpdateRnaSequencesAsyncTask,
            AutofillPartsAsyncTask,
            AutofillTranslationsAsyncTask,
            BulkRegisterEntitiesAsyncTask,
            BulkCreateDnaOligosAsyncTask,
            BulkUpdateDnaOligosAsyncTask,
            BulkCreateRnaOligosAsyncTask,
            BulkCreateAaSequencesAsyncTask,
            BulkCreateCustomEntitiesAsyncTask,
            BulkUpdateCustomEntitiesAsyncTask,
            BulkCreateContainersAsyncTask,
            BulkUpdateContainersAsyncTask,
            BulkUpdateAaSequencesAsyncTask,
            BulkUpdateRnaOligosAsyncTask,
            TransfersAsyncTask,
            AOPProcessOutputAsyncTask,
            AIGGenerateInputAsyncTask,
            ExportsAsyncTask,
            BulkCreateFeaturesAsyncTask,
            FindMatchingRegionsAsyncTask,
            FindMatchingRegionsDnaAsyncTask,
            UnknownType,
        ],
        NotFoundError,
    ]
]:
    """ Get a task by id """

    return (
        await asyncio_detailed(
            client=client,
            task_id=task_id,
        )
    ).parsed



from enum import Enum, IntEnum
from typing import Any, TypeVar

import attr


class NotPresentError(Exception):
    path: str
    message: str
    parent_name: str

    def __init__(self, parent: Any, path: str):
        self.path = path
        self.parent_name = type(parent).__name__
        self.message = f"Attempted to read '{self.parent_name}.{self.path}', which is Unset"
        super().__init__(self.message)


@attr.s(auto_attribs=True)
class UnknownType:
    """
    Represents a polymorphic type that this version of the SDK is not aware of.

    Oftentimes in e.g. listing endpoints, many different types of an entity can be returned.  As new features are added,
    new types can be returned from that same endpoint, but given that the SDK may not have been updated to know how to
    deserialize them, this would normally cause an error during deserialization.  This prevents normal program flow in
    cases which could be otherwise written defensively: none of the fields are programmatically accessible if
    deserialization fails, but programs may opt to handle errors or use the information they do know how to work with if
    serialization is made to be more 'nice.'
    """

    value: Any


T = TypeVar("T")


class Enums:
    class UnknownString(str, Enum):
        @staticmethod
        def known() -> bool:
            return False

        def __eq__(self, obj: Any) -> bool:
            return obj is self

        def __ne__(self, obj: Any) -> bool:
            return obj is not self

    class KnownString(str, Enum):
        @staticmethod
        def known() -> bool:
            return True

    class UnknownInt(IntEnum):
        @staticmethod
        def known() -> bool:
            return False

        def __eq__(self, obj: Any) -> bool:
            return obj is self

        def __ne__(self, obj: Any) -> bool:
            return obj is not self

    class KnownInt(IntEnum):
        @staticmethod
        def known() -> bool:
            return True



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="Primer")


@attr.s(auto_attribs=True, repr=False)
class Primer:
    """  """

    _bases: Union[Unset, str] = UNSET
    _bind_position: Union[Unset, int] = UNSET
    _color: Union[Unset, str] = UNSET
    _end: Union[Unset, int] = UNSET
    _name: Union[Unset, str] = UNSET
    _oligo_id: Union[Unset, str] = UNSET
    _overhang_length: Union[Unset, int] = UNSET
    _start: Union[Unset, int] = UNSET
    _strand: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("bases={}".format(repr(self._bases)))
        fields.append("bind_position={}".format(repr(self._bind_position)))
        fields.append("color={}".format(repr(self._color)))
        fields.append("end={}".format(repr(self._end)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("oligo_id={}".format(repr(self._oligo_id)))
        fields.append("overhang_length={}".format(repr(self._overhang_length)))
        fields.append("start={}".format(repr(self._start)))
        fields.append("strand={}".format(repr(self._strand)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "Primer({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        bases = self._bases
        bind_position = self._bind_position
        color = self._color
        end = self._end
        name = self._name
        oligo_id = self._oligo_id
        overhang_length = self._overhang_length
        start = self._start
        strand = self._strand

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if bases is not UNSET:
            field_dict["bases"] = bases
        if bind_position is not UNSET:
            field_dict["bindPosition"] = bind_position
        if color is not UNSET:
            field_dict["color"] = color
        if end is not UNSET:
            field_dict["end"] = end
        if name is not UNSET:
            field_dict["name"] = name
        if oligo_id is not UNSET:
            field_dict["oligoId"] = oligo_id
        if overhang_length is not UNSET:
            field_dict["overhangLength"] = overhang_length
        if start is not UNSET:
            field_dict["start"] = start
        if strand is not UNSET:
            field_dict["strand"] = strand

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_bases() -> Union[Unset, str]:
            bases = d.pop("bases")
            return bases

        try:
            bases = get_bases()
        except KeyError:
            if strict:
                raise
            bases = cast(Union[Unset, str], UNSET)

        def get_bind_position() -> Union[Unset, int]:
            bind_position = d.pop("bindPosition")
            return bind_position

        try:
            bind_position = get_bind_position()
        except KeyError:
            if strict:
                raise
            bind_position = cast(Union[Unset, int], UNSET)

        def get_color() -> Union[Unset, str]:
            color = d.pop("color")
            return color

        try:
            color = get_color()
        except KeyError:
            if strict:
                raise
            color = cast(Union[Unset, str], UNSET)

        def get_end() -> Union[Unset, int]:
            end = d.pop("end")
            return end

        try:
            end = get_end()
        except KeyError:
            if strict:
                raise
            end = cast(Union[Unset, int], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_oligo_id() -> Union[Unset, str]:
            oligo_id = d.pop("oligoId")
            return oligo_id

        try:
            oligo_id = get_oligo_id()
        except KeyError:
            if strict:
                raise
            oligo_id = cast(Union[Unset, str], UNSET)

        def get_overhang_length() -> Union[Unset, int]:
            overhang_length = d.pop("overhangLength")
            return overhang_length

        try:
            overhang_length = get_overhang_length()
        except KeyError:
            if strict:
                raise
            overhang_length = cast(Union[Unset, int], UNSET)

        def get_start() -> Union[Unset, int]:
            start = d.pop("start")
            return start

        try:
            start = get_start()
        except KeyError:
            if strict:
                raise
            start = cast(Union[Unset, int], UNSET)

        def get_strand() -> Union[Unset, int]:
            strand = d.pop("strand")
            return strand

        try:
            strand = get_strand()
        except KeyError:
            if strict:
                raise
            strand = cast(Union[Unset, int], UNSET)

        primer = cls(
            bases=bases,
            bind_position=bind_position,
            color=color,
            end=end,
            name=name,
            oligo_id=oligo_id,
            overhang_length=overhang_length,
            start=start,
            strand=strand,
        )

        primer.additional_properties = d
        return primer

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def bases(self) -> str:
        if isinstance(self._bases, Unset):
            raise NotPresentError(self, "bases")
        return self._bases

    @bases.setter
    def bases(self, value: str) -> None:
        self._bases = value

    @bases.deleter
    def bases(self) -> None:
        self._bases = UNSET

    @property
    def bind_position(self) -> int:
        if isinstance(self._bind_position, Unset):
            raise NotPresentError(self, "bind_position")
        return self._bind_position

    @bind_position.setter
    def bind_position(self, value: int) -> None:
        self._bind_position = value

    @bind_position.deleter
    def bind_position(self) -> None:
        self._bind_position = UNSET

    @property
    def color(self) -> str:
        if isinstance(self._color, Unset):
            raise NotPresentError(self, "color")
        return self._color

    @color.setter
    def color(self, value: str) -> None:
        self._color = value

    @color.deleter
    def color(self) -> None:
        self._color = UNSET

    @property
    def end(self) -> int:
        if isinstance(self._end, Unset):
            raise NotPresentError(self, "end")
        return self._end

    @end.setter
    def end(self, value: int) -> None:
        self._end = value

    @end.deleter
    def end(self) -> None:
        self._end = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def oligo_id(self) -> str:
        if isinstance(self._oligo_id, Unset):
            raise NotPresentError(self, "oligo_id")
        return self._oligo_id

    @oligo_id.setter
    def oligo_id(self, value: str) -> None:
        self._oligo_id = value

    @oligo_id.deleter
    def oligo_id(self) -> None:
        self._oligo_id = UNSET

    @property
    def overhang_length(self) -> int:
        if isinstance(self._overhang_length, Unset):
            raise NotPresentError(self, "overhang_length")
        return self._overhang_length

    @overhang_length.setter
    def overhang_length(self, value: int) -> None:
        self._overhang_length = value

    @overhang_length.deleter
    def overhang_length(self) -> None:
        self._overhang_length = UNSET

    @property
    def start(self) -> int:
        if isinstance(self._start, Unset):
            raise NotPresentError(self, "start")
        return self._start

    @start.setter
    def start(self, value: int) -> None:
        self._start = value

    @start.deleter
    def start(self) -> None:
        self._start = UNSET

    @property
    def strand(self) -> int:
        if isinstance(self._strand, Unset):
            raise NotPresentError(self, "strand")
        return self._strand

    @strand.setter
    def strand(self, value: int) -> None:
        self._strand = value

    @strand.deleter
    def strand(self) -> None:
        self._strand = UNSET



from typing import Union

from ..extensions import UnknownType
from ..models.form_barcode_value_reference import FormBarcodeValueReference
from ..models.form_field_value_reference import FormFieldValueReference
from ..models.form_raw_string_query_value_part import FormRawStringQueryValuePart

BarcodeQueryValuePart = Union[
    FormBarcodeValueReference, FormFieldValueReference, FormRawStringQueryValuePart, UnknownType
]



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="AutomationProgressStats")


@attr.s(auto_attribs=True, repr=False)
class AutomationProgressStats:
    """ Processing progress information. """

    _rows_failed: Union[Unset, int] = UNSET
    _rows_succeeded: Union[Unset, int] = UNSET
    _rows_unprocessed: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("rows_failed={}".format(repr(self._rows_failed)))
        fields.append("rows_succeeded={}".format(repr(self._rows_succeeded)))
        fields.append("rows_unprocessed={}".format(repr(self._rows_unprocessed)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "AutomationProgressStats({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        rows_failed = self._rows_failed
        rows_succeeded = self._rows_succeeded
        rows_unprocessed = self._rows_unprocessed

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if rows_failed is not UNSET:
            field_dict["rowsFailed"] = rows_failed
        if rows_succeeded is not UNSET:
            field_dict["rowsSucceeded"] = rows_succeeded
        if rows_unprocessed is not UNSET:
            field_dict["rowsUnprocessed"] = rows_unprocessed

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_rows_failed() -> Union[Unset, int]:
            rows_failed = d.pop("rowsFailed")
            return rows_failed

        try:
            rows_failed = get_rows_failed()
        except KeyError:
            if strict:
                raise
            rows_failed = cast(Union[Unset, int], UNSET)

        def get_rows_succeeded() -> Union[Unset, int]:
            rows_succeeded = d.pop("rowsSucceeded")
            return rows_succeeded

        try:
            rows_succeeded = get_rows_succeeded()
        except KeyError:
            if strict:
                raise
            rows_succeeded = cast(Union[Unset, int], UNSET)

        def get_rows_unprocessed() -> Union[Unset, int]:
            rows_unprocessed = d.pop("rowsUnprocessed")
            return rows_unprocessed

        try:
            rows_unprocessed = get_rows_unprocessed()
        except KeyError:
            if strict:
                raise
            rows_unprocessed = cast(Union[Unset, int], UNSET)

        automation_progress_stats = cls(
            rows_failed=rows_failed,
            rows_succeeded=rows_succeeded,
            rows_unprocessed=rows_unprocessed,
        )

        automation_progress_stats.additional_properties = d
        return automation_progress_stats

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def rows_failed(self) -> int:
        if isinstance(self._rows_failed, Unset):
            raise NotPresentError(self, "rows_failed")
        return self._rows_failed

    @rows_failed.setter
    def rows_failed(self, value: int) -> None:
        self._rows_failed = value

    @rows_failed.deleter
    def rows_failed(self) -> None:
        self._rows_failed = UNSET

    @property
    def rows_succeeded(self) -> int:
        if isinstance(self._rows_succeeded, Unset):
            raise NotPresentError(self, "rows_succeeded")
        return self._rows_succeeded

    @rows_succeeded.setter
    def rows_succeeded(self, value: int) -> None:
        self._rows_succeeded = value

    @rows_succeeded.deleter
    def rows_succeeded(self) -> None:
        self._rows_succeeded = UNSET

    @property
    def rows_unprocessed(self) -> int:
        if isinstance(self._rows_unprocessed, Unset):
            raise NotPresentError(self, "rows_unprocessed")
        return self._rows_unprocessed

    @rows_unprocessed.setter
    def rows_unprocessed(self, value: int) -> None:
        self._rows_unprocessed = value

    @rows_unprocessed.deleter
    def rows_unprocessed(self) -> None:
        self._rows_unprocessed = UNSET



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.async_task_link import AsyncTaskLink
from ...models.bad_request_error import BadRequestError
from ...models.mixtures_bulk_update_request import MixturesBulkUpdateRequest
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    json_body: MixturesBulkUpdateRequest,
) -> Dict[str, Any]:
    url = "{}/mixtures:bulk-update".format(client.base_url)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    if response.status_code == 202:
        response_202 = AsyncTaskLink.from_dict(response.json(), strict=False)

        return response_202
    if response.status_code == 400:
        response_400 = BadRequestError.from_dict(response.json(), strict=False)

        return response_400
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: MixturesBulkUpdateRequest,
) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = client.httpx_client.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    json_body: MixturesBulkUpdateRequest,
) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    """Update multiple mixtures.
    To change the parent mixture on your specified mixtures, set the parent mixture field and specify the desired final state for your ingredients.
    Benchling will recognize that any ingredients you specify that match ingredients on the parent mixtures (based on component entity) are inherited. This can be seen on the returned `ingredients[i].hasParent` attribute.
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    json_body: MixturesBulkUpdateRequest,
) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    json_body: MixturesBulkUpdateRequest,
) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    """Update multiple mixtures.
    To change the parent mixture on your specified mixtures, set the parent mixture field and specify the desired final state for your ingredients.
    Benchling will recognize that any ingredients you specify that match ingredients on the parent mixtures (based on component entity) are inherited. This can be seen on the returned `ingredients[i].hasParent` attribute.
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
        )
    ).parsed



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.app_config_item_boolean_create import AppConfigItemBooleanCreate
from ..models.app_config_item_date_create import AppConfigItemDateCreate
from ..models.app_config_item_datetime_create import AppConfigItemDatetimeCreate
from ..models.app_config_item_float_create import AppConfigItemFloatCreate
from ..models.app_config_item_generic_create import AppConfigItemGenericCreate
from ..models.app_config_item_integer_create import AppConfigItemIntegerCreate
from ..models.app_config_item_json_create import AppConfigItemJsonCreate
from ..types import UNSET, Unset

T = TypeVar("T", bound="AppConfigItemsBulkCreateRequest")


@attr.s(auto_attribs=True, repr=False)
class AppConfigItemsBulkCreateRequest:
    """  """

    _app_configuration_items: List[
        Union[
            AppConfigItemGenericCreate,
            AppConfigItemBooleanCreate,
            AppConfigItemIntegerCreate,
            AppConfigItemFloatCreate,
            AppConfigItemDateCreate,
            AppConfigItemDatetimeCreate,
            AppConfigItemJsonCreate,
            UnknownType,
        ]
    ]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("app_configuration_items={}".format(repr(self._app_configuration_items)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "AppConfigItemsBulkCreateRequest({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        app_configuration_items = []
        for app_configuration_items_item_data in self._app_configuration_items:
            if isinstance(app_configuration_items_item_data, UnknownType):
                app_configuration_items_item = app_configuration_items_item_data.value
            elif isinstance(app_configuration_items_item_data, AppConfigItemGenericCreate):
                app_configuration_items_item = app_configuration_items_item_data.to_dict()

            elif isinstance(app_configuration_items_item_data, AppConfigItemBooleanCreate):
                app_configuration_items_item = app_configuration_items_item_data.to_dict()

            elif isinstance(app_configuration_items_item_data, AppConfigItemIntegerCreate):
                app_configuration_items_item = app_configuration_items_item_data.to_dict()

            elif isinstance(app_configuration_items_item_data, AppConfigItemFloatCreate):
                app_configuration_items_item = app_configuration_items_item_data.to_dict()

            elif isinstance(app_configuration_items_item_data, AppConfigItemDateCreate):
                app_configuration_items_item = app_configuration_items_item_data.to_dict()

            elif isinstance(app_configuration_items_item_data, AppConfigItemDatetimeCreate):
                app_configuration_items_item = app_configuration_items_item_data.to_dict()

            else:
                app_configuration_items_item = app_configuration_items_item_data.to_dict()

            app_configuration_items.append(app_configuration_items_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if app_configuration_items is not UNSET:
            field_dict["appConfigurationItems"] = app_configuration_items

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_app_configuration_items() -> List[
            Union[
                AppConfigItemGenericCreate,
                AppConfigItemBooleanCreate,
                AppConfigItemIntegerCreate,
                AppConfigItemFloatCreate,
                AppConfigItemDateCreate,
                AppConfigItemDatetimeCreate,
                AppConfigItemJsonCreate,
                UnknownType,
            ]
        ]:
            app_configuration_items = []
            _app_configuration_items = d.pop("appConfigurationItems")
            for app_configuration_items_item_data in _app_configuration_items:

                def _parse_app_configuration_items_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[
                    AppConfigItemGenericCreate,
                    AppConfigItemBooleanCreate,
                    AppConfigItemIntegerCreate,
                    AppConfigItemFloatCreate,
                    AppConfigItemDateCreate,
                    AppConfigItemDatetimeCreate,
                    AppConfigItemJsonCreate,
                    UnknownType,
                ]:
                    app_configuration_items_item: Union[
                        AppConfigItemGenericCreate,
                        AppConfigItemBooleanCreate,
                        AppConfigItemIntegerCreate,
                        AppConfigItemFloatCreate,
                        AppConfigItemDateCreate,
                        AppConfigItemDatetimeCreate,
                        AppConfigItemJsonCreate,
                        UnknownType,
                    ]
                    discriminator_value: str = cast(str, data.get("type"))
                    if discriminator_value is not None:
                        app_config_item_create: Union[
                            AppConfigItemGenericCreate,
                            AppConfigItemBooleanCreate,
                            AppConfigItemIntegerCreate,
                            AppConfigItemFloatCreate,
                            AppConfigItemDateCreate,
                            AppConfigItemDatetimeCreate,
                            AppConfigItemJsonCreate,
                            UnknownType,
                        ]
                        if discriminator_value == "aa_sequence":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "boolean":
                            app_config_item_create = AppConfigItemBooleanCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "box":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "box_schema":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "container":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "container_schema":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "custom_entity":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "date":
                            app_config_item_create = AppConfigItemDateCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "datetime":
                            app_config_item_create = AppConfigItemDatetimeCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "dna_oligo":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "dna_sequence":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "dropdown":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "dropdown_option":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "entity_schema":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "entry":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "entry_schema":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "field":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "float":
                            app_config_item_create = AppConfigItemFloatCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "folder":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "integer":
                            app_config_item_create = AppConfigItemIntegerCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "json":
                            app_config_item_create = AppConfigItemJsonCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "location":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "location_schema":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "mixture":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "molecule":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "plate":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "plate_schema":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "project":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "registry":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "request_schema":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "result_schema":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "rna_oligo":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "rna_sequence":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "run_schema":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "secure_text":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "text":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "workflow_task_schema":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "workflow_task_status":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create
                        if discriminator_value == "worklist":
                            app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=False)

                            return app_config_item_create

                        return UnknownType(value=data)
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item_create = AppConfigItemGenericCreate.from_dict(data, strict=True)

                        return app_config_item_create
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item_create = AppConfigItemBooleanCreate.from_dict(data, strict=True)

                        return app_config_item_create
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item_create = AppConfigItemIntegerCreate.from_dict(data, strict=True)

                        return app_config_item_create
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item_create = AppConfigItemFloatCreate.from_dict(data, strict=True)

                        return app_config_item_create
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item_create = AppConfigItemDateCreate.from_dict(data, strict=True)

                        return app_config_item_create
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item_create = AppConfigItemDatetimeCreate.from_dict(data, strict=True)

                        return app_config_item_create
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        app_config_item_create = AppConfigItemJsonCreate.from_dict(data, strict=True)

                        return app_config_item_create
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                app_configuration_items_item = _parse_app_configuration_items_item(
                    app_configuration_items_item_data
                )

                app_configuration_items.append(app_configuration_items_item)

            return app_configuration_items

        try:
            app_configuration_items = get_app_configuration_items()
        except KeyError:
            if strict:
                raise
            app_configuration_items = cast(
                List[
                    Union[
                        AppConfigItemGenericCreate,
                        AppConfigItemBooleanCreate,
                        AppConfigItemIntegerCreate,
                        AppConfigItemFloatCreate,
                        AppConfigItemDateCreate,
                        AppConfigItemDatetimeCreate,
                        AppConfigItemJsonCreate,
                        UnknownType,
                    ]
                ],
                UNSET,
            )

        app_config_items_bulk_create_request = cls(
            app_configuration_items=app_configuration_items,
        )

        app_config_items_bulk_create_request.additional_properties = d
        return app_config_items_bulk_create_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def app_configuration_items(
        self,
    ) -> List[
        Union[
            AppConfigItemGenericCreate,
            AppConfigItemBooleanCreate,
            AppConfigItemIntegerCreate,
            AppConfigItemFloatCreate,
            AppConfigItemDateCreate,
            AppConfigItemDatetimeCreate,
            AppConfigItemJsonCreate,
            UnknownType,
        ]
    ]:
        if isinstance(self._app_configuration_items, Unset):
            raise NotPresentError(self, "app_configuration_items")
        return self._app_configuration_items

    @app_configuration_items.setter
    def app_configuration_items(
        self,
        value: List[
            Union[
                AppConfigItemGenericCreate,
                AppConfigItemBooleanCreate,
                AppConfigItemIntegerCreate,
                AppConfigItemFloatCreate,
                AppConfigItemDateCreate,
                AppConfigItemDatetimeCreate,
                AppConfigItemJsonCreate,
                UnknownType,
            ]
        ],
    ) -> None:
        self._app_configuration_items = value



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..models.custom_fields import CustomFields
from ..models.entry_schema import EntrySchema
from ..models.fields import Fields
from ..models.stage_entry_review_record import StageEntryReviewRecord
from ..models.user_summary import UserSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="StageEntry")


@attr.s(auto_attribs=True, repr=False)
class StageEntry:
    """ A notebook entry used for execution of one or more stage runs in a legacy workflow. """

    _api_url: Union[Unset, str] = UNSET
    _authors: Union[Unset, List[UserSummary]] = UNSET
    _created_at: Union[Unset, datetime.datetime] = UNSET
    _creator: Union[Unset, UserSummary] = UNSET
    _custom_fields: Union[Unset, CustomFields] = UNSET
    _display_id: Union[Unset, str] = UNSET
    _fields: Union[Unset, Fields] = UNSET
    _folder_id: Union[Unset, str] = UNSET
    _id: Union[Unset, str] = UNSET
    _modified_at: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    _review_record: Union[Unset, None, StageEntryReviewRecord] = UNSET
    _schema: Union[Unset, None, EntrySchema] = UNSET
    _web_url: Union[Unset, str] = UNSET
    _workflow_id: Union[Unset, str] = UNSET
    _workflow_stage_id: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("api_url={}".format(repr(self._api_url)))
        fields.append("authors={}".format(repr(self._authors)))
        fields.append("created_at={}".format(repr(self._created_at)))
        fields.append("creator={}".format(repr(self._creator)))
        fields.append("custom_fields={}".format(repr(self._custom_fields)))
        fields.append("display_id={}".format(repr(self._display_id)))
        fields.append("fields={}".format(repr(self._fields)))
        fields.append("folder_id={}".format(repr(self._folder_id)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("modified_at={}".format(repr(self._modified_at)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("review_record={}".format(repr(self._review_record)))
        fields.append("schema={}".format(repr(self._schema)))
        fields.append("web_url={}".format(repr(self._web_url)))
        fields.append("workflow_id={}".format(repr(self._workflow_id)))
        fields.append("workflow_stage_id={}".format(repr(self._workflow_stage_id)))
        return "StageEntry({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        api_url = self._api_url
        authors: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._authors, Unset):
            authors = []
            for authors_item_data in self._authors:
                authors_item = authors_item_data.to_dict()

                authors.append(authors_item)

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self._created_at, Unset):
            created_at = self._created_at.isoformat()

        creator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._creator, Unset):
            creator = self._creator.to_dict()

        custom_fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._custom_fields, Unset):
            custom_fields = self._custom_fields.to_dict()

        display_id = self._display_id
        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._fields, Unset):
            fields = self._fields.to_dict()

        folder_id = self._folder_id
        id = self._id
        modified_at = self._modified_at
        name = self._name
        review_record: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._review_record, Unset):
            review_record = self._review_record.to_dict() if self._review_record else None

        schema: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._schema, Unset):
            schema = self._schema.to_dict() if self._schema else None

        web_url = self._web_url
        workflow_id = self._workflow_id
        workflow_stage_id = self._workflow_stage_id

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if api_url is not UNSET:
            field_dict["apiURL"] = api_url
        if authors is not UNSET:
            field_dict["authors"] = authors
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if creator is not UNSET:
            field_dict["creator"] = creator
        if custom_fields is not UNSET:
            field_dict["customFields"] = custom_fields
        if display_id is not UNSET:
            field_dict["displayId"] = display_id
        if fields is not UNSET:
            field_dict["fields"] = fields
        if folder_id is not UNSET:
            field_dict["folderId"] = folder_id
        if id is not UNSET:
            field_dict["id"] = id
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if name is not UNSET:
            field_dict["name"] = name
        if review_record is not UNSET:
            field_dict["reviewRecord"] = review_record
        if schema is not UNSET:
            field_dict["schema"] = schema
        if web_url is not UNSET:
            field_dict["webURL"] = web_url
        if workflow_id is not UNSET:
            field_dict["workflowId"] = workflow_id
        if workflow_stage_id is not UNSET:
            field_dict["workflowStageId"] = workflow_stage_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_api_url() -> Union[Unset, str]:
            api_url = d.pop("apiURL")
            return api_url

        try:
            api_url = get_api_url()
        except KeyError:
            if strict:
                raise
            api_url = cast(Union[Unset, str], UNSET)

        def get_authors() -> Union[Unset, List[UserSummary]]:
            authors = []
            _authors = d.pop("authors")
            for authors_item_data in _authors or []:
                authors_item = UserSummary.from_dict(authors_item_data, strict=False)

                authors.append(authors_item)

            return authors

        try:
            authors = get_authors()
        except KeyError:
            if strict:
                raise
            authors = cast(Union[Unset, List[UserSummary]], UNSET)

        def get_created_at() -> Union[Unset, datetime.datetime]:
            created_at: Union[Unset, datetime.datetime] = UNSET
            _created_at = d.pop("createdAt")
            if _created_at is not None and not isinstance(_created_at, Unset):
                created_at = isoparse(cast(str, _created_at))

            return created_at

        try:
            created_at = get_created_at()
        except KeyError:
            if strict:
                raise
            created_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_creator() -> Union[Unset, UserSummary]:
            creator: Union[Unset, Union[Unset, UserSummary]] = UNSET
            _creator = d.pop("creator")

            if not isinstance(_creator, Unset):
                creator = UserSummary.from_dict(_creator)

            return creator

        try:
            creator = get_creator()
        except KeyError:
            if strict:
                raise
            creator = cast(Union[Unset, UserSummary], UNSET)

        def get_custom_fields() -> Union[Unset, CustomFields]:
            custom_fields: Union[Unset, Union[Unset, CustomFields]] = UNSET
            _custom_fields = d.pop("customFields")

            if not isinstance(_custom_fields, Unset):
                custom_fields = CustomFields.from_dict(_custom_fields)

            return custom_fields

        try:
            custom_fields = get_custom_fields()
        except KeyError:
            if strict:
                raise
            custom_fields = cast(Union[Unset, CustomFields], UNSET)

        def get_display_id() -> Union[Unset, str]:
            display_id = d.pop("displayId")
            return display_id

        try:
            display_id = get_display_id()
        except KeyError:
            if strict:
                raise
            display_id = cast(Union[Unset, str], UNSET)

        def get_fields() -> Union[Unset, Fields]:
            fields: Union[Unset, Union[Unset, Fields]] = UNSET
            _fields = d.pop("fields")

            if not isinstance(_fields, Unset):
                fields = Fields.from_dict(_fields)

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Unset, Fields], UNSET)

        def get_folder_id() -> Union[Unset, str]:
            folder_id = d.pop("folderId")
            return folder_id

        try:
            folder_id = get_folder_id()
        except KeyError:
            if strict:
                raise
            folder_id = cast(Union[Unset, str], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_modified_at() -> Union[Unset, str]:
            modified_at = d.pop("modifiedAt")
            return modified_at

        try:
            modified_at = get_modified_at()
        except KeyError:
            if strict:
                raise
            modified_at = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_review_record() -> Union[Unset, None, StageEntryReviewRecord]:
            review_record = None
            _review_record = d.pop("reviewRecord")

            if _review_record is not None and not isinstance(_review_record, Unset):
                review_record = StageEntryReviewRecord.from_dict(_review_record)

            return review_record

        try:
            review_record = get_review_record()
        except KeyError:
            if strict:
                raise
            review_record = cast(Union[Unset, None, StageEntryReviewRecord], UNSET)

        def get_schema() -> Union[Unset, None, EntrySchema]:
            schema = None
            _schema = d.pop("schema")

            if _schema is not None and not isinstance(_schema, Unset):
                schema = EntrySchema.from_dict(_schema)

            return schema

        try:
            schema = get_schema()
        except KeyError:
            if strict:
                raise
            schema = cast(Union[Unset, None, EntrySchema], UNSET)

        def get_web_url() -> Union[Unset, str]:
            web_url = d.pop("webURL")
            return web_url

        try:
            web_url = get_web_url()
        except KeyError:
            if strict:
                raise
            web_url = cast(Union[Unset, str], UNSET)

        def get_workflow_id() -> Union[Unset, str]:
            workflow_id = d.pop("workflowId")
            return workflow_id

        try:
            workflow_id = get_workflow_id()
        except KeyError:
            if strict:
                raise
            workflow_id = cast(Union[Unset, str], UNSET)

        def get_workflow_stage_id() -> Union[Unset, str]:
            workflow_stage_id = d.pop("workflowStageId")
            return workflow_stage_id

        try:
            workflow_stage_id = get_workflow_stage_id()
        except KeyError:
            if strict:
                raise
            workflow_stage_id = cast(Union[Unset, str], UNSET)

        stage_entry = cls(
            api_url=api_url,
            authors=authors,
            created_at=created_at,
            creator=creator,
            custom_fields=custom_fields,
            display_id=display_id,
            fields=fields,
            folder_id=folder_id,
            id=id,
            modified_at=modified_at,
            name=name,
            review_record=review_record,
            schema=schema,
            web_url=web_url,
            workflow_id=workflow_id,
            workflow_stage_id=workflow_stage_id,
        )

        return stage_entry

    @property
    def api_url(self) -> str:
        """ The canonical url of the Stage Entry in the API. """
        if isinstance(self._api_url, Unset):
            raise NotPresentError(self, "api_url")
        return self._api_url

    @api_url.setter
    def api_url(self, value: str) -> None:
        self._api_url = value

    @api_url.deleter
    def api_url(self) -> None:
        self._api_url = UNSET

    @property
    def authors(self) -> List[UserSummary]:
        """Array of UserSummary Resources of the authors of the stage entry. This defaults to the creator but can be manually changed."""
        if isinstance(self._authors, Unset):
            raise NotPresentError(self, "authors")
        return self._authors

    @authors.setter
    def authors(self, value: List[UserSummary]) -> None:
        self._authors = value

    @authors.deleter
    def authors(self) -> None:
        self._authors = UNSET

    @property
    def created_at(self) -> datetime.datetime:
        """ DateTime the stage entry was created at """
        if isinstance(self._created_at, Unset):
            raise NotPresentError(self, "created_at")
        return self._created_at

    @created_at.setter
    def created_at(self, value: datetime.datetime) -> None:
        self._created_at = value

    @created_at.deleter
    def created_at(self) -> None:
        self._created_at = UNSET

    @property
    def creator(self) -> UserSummary:
        if isinstance(self._creator, Unset):
            raise NotPresentError(self, "creator")
        return self._creator

    @creator.setter
    def creator(self, value: UserSummary) -> None:
        self._creator = value

    @creator.deleter
    def creator(self) -> None:
        self._creator = UNSET

    @property
    def custom_fields(self) -> CustomFields:
        if isinstance(self._custom_fields, Unset):
            raise NotPresentError(self, "custom_fields")
        return self._custom_fields

    @custom_fields.setter
    def custom_fields(self, value: CustomFields) -> None:
        self._custom_fields = value

    @custom_fields.deleter
    def custom_fields(self) -> None:
        self._custom_fields = UNSET

    @property
    def display_id(self) -> str:
        """ User-friendly ID of the stage entry """
        if isinstance(self._display_id, Unset):
            raise NotPresentError(self, "display_id")
        return self._display_id

    @display_id.setter
    def display_id(self, value: str) -> None:
        self._display_id = value

    @display_id.deleter
    def display_id(self) -> None:
        self._display_id = UNSET

    @property
    def fields(self) -> Fields:
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Fields) -> None:
        self._fields = value

    @fields.deleter
    def fields(self) -> None:
        self._fields = UNSET

    @property
    def folder_id(self) -> str:
        """ ID of the folder that contains the stage entry """
        if isinstance(self._folder_id, Unset):
            raise NotPresentError(self, "folder_id")
        return self._folder_id

    @folder_id.setter
    def folder_id(self, value: str) -> None:
        self._folder_id = value

    @folder_id.deleter
    def folder_id(self) -> None:
        self._folder_id = UNSET

    @property
    def id(self) -> str:
        """ ID of the stage entry """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def modified_at(self) -> str:
        """ DateTime the stage entry was last modified """
        if isinstance(self._modified_at, Unset):
            raise NotPresentError(self, "modified_at")
        return self._modified_at

    @modified_at.setter
    def modified_at(self, value: str) -> None:
        self._modified_at = value

    @modified_at.deleter
    def modified_at(self) -> None:
        self._modified_at = UNSET

    @property
    def name(self) -> str:
        """ Title of the stage entry """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def review_record(self) -> Optional[StageEntryReviewRecord]:
        """ Review record if set """
        if isinstance(self._review_record, Unset):
            raise NotPresentError(self, "review_record")
        return self._review_record

    @review_record.setter
    def review_record(self, value: Optional[StageEntryReviewRecord]) -> None:
        self._review_record = value

    @review_record.deleter
    def review_record(self) -> None:
        self._review_record = UNSET

    @property
    def schema(self) -> Optional[EntrySchema]:
        """ Entry schema """
        if isinstance(self._schema, Unset):
            raise NotPresentError(self, "schema")
        return self._schema

    @schema.setter
    def schema(self, value: Optional[EntrySchema]) -> None:
        self._schema = value

    @schema.deleter
    def schema(self) -> None:
        self._schema = UNSET

    @property
    def web_url(self) -> str:
        """ URL of the stage entry """
        if isinstance(self._web_url, Unset):
            raise NotPresentError(self, "web_url")
        return self._web_url

    @web_url.setter
    def web_url(self, value: str) -> None:
        self._web_url = value

    @web_url.deleter
    def web_url(self) -> None:
        self._web_url = UNSET

    @property
    def workflow_id(self) -> str:
        """ ID of the parent workflow """
        if isinstance(self._workflow_id, Unset):
            raise NotPresentError(self, "workflow_id")
        return self._workflow_id

    @workflow_id.setter
    def workflow_id(self, value: str) -> None:
        self._workflow_id = value

    @workflow_id.deleter
    def workflow_id(self) -> None:
        self._workflow_id = UNSET

    @property
    def workflow_stage_id(self) -> str:
        """ ID of the associated workflow stage """
        if isinstance(self._workflow_stage_id, Unset):
            raise NotPresentError(self, "workflow_stage_id")
        return self._workflow_stage_id

    @workflow_stage_id.setter
    def workflow_stage_id(self, value: str) -> None:
        self._workflow_stage_id = value

    @workflow_stage_id.deleter
    def workflow_stage_id(self) -> None:
        self._workflow_stage_id = UNSET



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..models.session_message import SessionMessage
from ..models.session_status import SessionStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="Session")


@attr.s(auto_attribs=True, repr=False)
class Session:
    """  """

    _app_id: Union[Unset, str] = UNSET
    _created_at: Union[Unset, datetime.datetime] = UNSET
    _id: Union[Unset, str] = UNSET
    _messages: Union[Unset, None, List[SessionMessage]] = UNSET
    _modified_at: Union[Unset, datetime.datetime] = UNSET
    _name: Union[Unset, str] = UNSET
    _status: Union[Unset, SessionStatus] = UNSET
    _timeout_seconds: Union[Unset, int] = UNSET

    def __repr__(self):
        fields = []
        fields.append("app_id={}".format(repr(self._app_id)))
        fields.append("created_at={}".format(repr(self._created_at)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("messages={}".format(repr(self._messages)))
        fields.append("modified_at={}".format(repr(self._modified_at)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("status={}".format(repr(self._status)))
        fields.append("timeout_seconds={}".format(repr(self._timeout_seconds)))
        return "Session({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        app_id = self._app_id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self._created_at, Unset):
            created_at = self._created_at.isoformat()

        id = self._id
        messages: Union[Unset, None, List[Any]] = UNSET
        if not isinstance(self._messages, Unset):
            if self._messages is None:
                messages = None
            else:
                messages = []
                for messages_item_data in self._messages:
                    messages_item = messages_item_data.to_dict()

                    messages.append(messages_item)

        modified_at: Union[Unset, str] = UNSET
        if not isinstance(self._modified_at, Unset):
            modified_at = self._modified_at.isoformat()

        name = self._name
        status: Union[Unset, int] = UNSET
        if not isinstance(self._status, Unset):
            status = self._status.value

        timeout_seconds = self._timeout_seconds

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if app_id is not UNSET:
            field_dict["appId"] = app_id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if id is not UNSET:
            field_dict["id"] = id
        if messages is not UNSET:
            field_dict["messages"] = messages
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if name is not UNSET:
            field_dict["name"] = name
        if status is not UNSET:
            field_dict["status"] = status
        if timeout_seconds is not UNSET:
            field_dict["timeoutSeconds"] = timeout_seconds

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_app_id() -> Union[Unset, str]:
            app_id = d.pop("appId")
            return app_id

        try:
            app_id = get_app_id()
        except KeyError:
            if strict:
                raise
            app_id = cast(Union[Unset, str], UNSET)

        def get_created_at() -> Union[Unset, datetime.datetime]:
            created_at: Union[Unset, datetime.datetime] = UNSET
            _created_at = d.pop("createdAt")
            if _created_at is not None and not isinstance(_created_at, Unset):
                created_at = isoparse(cast(str, _created_at))

            return created_at

        try:
            created_at = get_created_at()
        except KeyError:
            if strict:
                raise
            created_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_messages() -> Union[Unset, None, List[SessionMessage]]:
            messages = []
            _messages = d.pop("messages")
            for messages_item_data in _messages or []:
                messages_item = SessionMessage.from_dict(messages_item_data, strict=False)

                messages.append(messages_item)

            return messages

        try:
            messages = get_messages()
        except KeyError:
            if strict:
                raise
            messages = cast(Union[Unset, None, List[SessionMessage]], UNSET)

        def get_modified_at() -> Union[Unset, datetime.datetime]:
            modified_at: Union[Unset, datetime.datetime] = UNSET
            _modified_at = d.pop("modifiedAt")
            if _modified_at is not None and not isinstance(_modified_at, Unset):
                modified_at = isoparse(cast(str, _modified_at))

            return modified_at

        try:
            modified_at = get_modified_at()
        except KeyError:
            if strict:
                raise
            modified_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_status() -> Union[Unset, SessionStatus]:
            status = UNSET
            _status = d.pop("status")
            if _status is not None and _status is not UNSET:
                try:
                    status = SessionStatus(_status)
                except ValueError:
                    status = SessionStatus.of_unknown(_status)

            return status

        try:
            status = get_status()
        except KeyError:
            if strict:
                raise
            status = cast(Union[Unset, SessionStatus], UNSET)

        def get_timeout_seconds() -> Union[Unset, int]:
            timeout_seconds = d.pop("timeoutSeconds")
            return timeout_seconds

        try:
            timeout_seconds = get_timeout_seconds()
        except KeyError:
            if strict:
                raise
            timeout_seconds = cast(Union[Unset, int], UNSET)

        session = cls(
            app_id=app_id,
            created_at=created_at,
            id=id,
            messages=messages,
            modified_at=modified_at,
            name=name,
            status=status,
            timeout_seconds=timeout_seconds,
        )

        return session

    @property
    def app_id(self) -> str:
        if isinstance(self._app_id, Unset):
            raise NotPresentError(self, "app_id")
        return self._app_id

    @app_id.setter
    def app_id(self, value: str) -> None:
        self._app_id = value

    @app_id.deleter
    def app_id(self) -> None:
        self._app_id = UNSET

    @property
    def created_at(self) -> datetime.datetime:
        if isinstance(self._created_at, Unset):
            raise NotPresentError(self, "created_at")
        return self._created_at

    @created_at.setter
    def created_at(self, value: datetime.datetime) -> None:
        self._created_at = value

    @created_at.deleter
    def created_at(self) -> None:
        self._created_at = UNSET

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def messages(self) -> Optional[List[SessionMessage]]:
        """An array of `SessionMessage` describing the current session state."""
        if isinstance(self._messages, Unset):
            raise NotPresentError(self, "messages")
        return self._messages

    @messages.setter
    def messages(self, value: Optional[List[SessionMessage]]) -> None:
        self._messages = value

    @messages.deleter
    def messages(self) -> None:
        self._messages = UNSET

    @property
    def modified_at(self) -> datetime.datetime:
        if isinstance(self._modified_at, Unset):
            raise NotPresentError(self, "modified_at")
        return self._modified_at

    @modified_at.setter
    def modified_at(self, value: datetime.datetime) -> None:
        self._modified_at = value

    @modified_at.deleter
    def modified_at(self) -> None:
        self._modified_at = UNSET

    @property
    def name(self) -> str:
        """ A brief description of the app's actions for users. Length must be between 3-100 chars. It becomes immutable once a value is set. """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def status(self) -> SessionStatus:
        """All possible values of a Session's status, including system-updated and user-updated values."""
        if isinstance(self._status, Unset):
            raise NotPresentError(self, "status")
        return self._status

    @status.setter
    def status(self, value: SessionStatus) -> None:
        self._status = value

    @status.deleter
    def status(self) -> None:
        self._status = UNSET

    @property
    def timeout_seconds(self) -> int:
        """Timeout in seconds, a value between 1 second and 30 days. Once set, it can only be increased, not decreased."""
        if isinstance(self._timeout_seconds, Unset):
            raise NotPresentError(self, "timeout_seconds")
        return self._timeout_seconds

    @timeout_seconds.setter
    def timeout_seconds(self, value: int) -> None:
        self._timeout_seconds = value

    @timeout_seconds.deleter
    def timeout_seconds(self) -> None:
        self._timeout_seconds = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.aa_sequence_bulk_create import AaSequenceBulkCreate
from ..types import UNSET, Unset

T = TypeVar("T", bound="AaSequencesBulkCreateRequest")


@attr.s(auto_attribs=True, repr=False)
class AaSequencesBulkCreateRequest:
    """  """

    _aa_sequences: Union[Unset, List[AaSequenceBulkCreate]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("aa_sequences={}".format(repr(self._aa_sequences)))
        return "AaSequencesBulkCreateRequest({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        aa_sequences: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._aa_sequences, Unset):
            aa_sequences = []
            for aa_sequences_item_data in self._aa_sequences:
                aa_sequences_item = aa_sequences_item_data.to_dict()

                aa_sequences.append(aa_sequences_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if aa_sequences is not UNSET:
            field_dict["aaSequences"] = aa_sequences

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_aa_sequences() -> Union[Unset, List[AaSequenceBulkCreate]]:
            aa_sequences = []
            _aa_sequences = d.pop("aaSequences")
            for aa_sequences_item_data in _aa_sequences or []:
                aa_sequences_item = AaSequenceBulkCreate.from_dict(aa_sequences_item_data, strict=False)

                aa_sequences.append(aa_sequences_item)

            return aa_sequences

        try:
            aa_sequences = get_aa_sequences()
        except KeyError:
            if strict:
                raise
            aa_sequences = cast(Union[Unset, List[AaSequenceBulkCreate]], UNSET)

        aa_sequences_bulk_create_request = cls(
            aa_sequences=aa_sequences,
        )

        return aa_sequences_bulk_create_request

    @property
    def aa_sequences(self) -> List[AaSequenceBulkCreate]:
        if isinstance(self._aa_sequences, Unset):
            raise NotPresentError(self, "aa_sequences")
        return self._aa_sequences

    @aa_sequences.setter
    def aa_sequences(self, value: List[AaSequenceBulkCreate]) -> None:
        self._aa_sequences = value

    @aa_sequences.deleter
    def aa_sequences(self) -> None:
        self._aa_sequences = UNSET



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class BarcodeScanFormInstanceProviderMode(Enums.KnownString):
    MULTIPLE_SCAN = "MULTIPLE_SCAN"
    SINGLE_SCAN = "SINGLE_SCAN"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "BarcodeScanFormInstanceProviderMode":
        if not isinstance(val, str):
            raise ValueError(
                f"Value of BarcodeScanFormInstanceProviderMode must be a string (encountered: {val})"
            )
        newcls = Enum("BarcodeScanFormInstanceProviderMode", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(BarcodeScanFormInstanceProviderMode, getattr(newcls, "_UNKNOWN"))



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.fields import Fields
from ..types import UNSET, Unset

T = TypeVar("T", bound="LocationUpdate")


@attr.s(auto_attribs=True, repr=False)
class LocationUpdate:
    """  """

    _fields: Union[Unset, Fields] = UNSET
    _name: Union[Unset, str] = UNSET
    _parent_storage_id: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("fields={}".format(repr(self._fields)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("parent_storage_id={}".format(repr(self._parent_storage_id)))
        return "LocationUpdate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._fields, Unset):
            fields = self._fields.to_dict()

        name = self._name
        parent_storage_id = self._parent_storage_id

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if fields is not UNSET:
            field_dict["fields"] = fields
        if name is not UNSET:
            field_dict["name"] = name
        if parent_storage_id is not UNSET:
            field_dict["parentStorageId"] = parent_storage_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_fields() -> Union[Unset, Fields]:
            fields: Union[Unset, Union[Unset, Fields]] = UNSET
            _fields = d.pop("fields")

            if not isinstance(_fields, Unset):
                fields = Fields.from_dict(_fields)

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Unset, Fields], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_parent_storage_id() -> Union[Unset, str]:
            parent_storage_id = d.pop("parentStorageId")
            return parent_storage_id

        try:
            parent_storage_id = get_parent_storage_id()
        except KeyError:
            if strict:
                raise
            parent_storage_id = cast(Union[Unset, str], UNSET)

        location_update = cls(
            fields=fields,
            name=name,
            parent_storage_id=parent_storage_id,
        )

        return location_update

    @property
    def fields(self) -> Fields:
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Fields) -> None:
        self._fields = value

    @fields.deleter
    def fields(self) -> None:
        self._fields = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def parent_storage_id(self) -> str:
        if isinstance(self._parent_storage_id, Unset):
            raise NotPresentError(self, "parent_storage_id")
        return self._parent_storage_id

    @parent_storage_id.setter
    def parent_storage_id(self, value: str) -> None:
        self._parent_storage_id = value

    @parent_storage_id.deleter
    def parent_storage_id(self) -> None:
        self._parent_storage_id = UNSET



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...extensions import UnknownType
from ...models.analysis_dataset_ids import AnalysisDatasetIds
from ...models.analysis_step_dataset_data import AnalysisStepDatasetData
from ...models.analysis_step_file_data import AnalysisStepFileData
from ...models.bad_request_error import BadRequestError
from ...models.file_ids import FileIds
from ...models.forbidden_error import ForbiddenError
from ...models.not_found_error import NotFoundError
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    analysis_step_data_id: str,
    json_body: Union[FileIds, AnalysisDatasetIds, UnknownType],
) -> Dict[str, Any]:
    url = "{}/analysis-step-data/{analysis_step_data_id}".format(
        client.base_url, analysis_step_data_id=analysis_step_data_id
    )

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    if isinstance(json_body, UnknownType):
        json_json_body = json_body.value
    elif isinstance(json_body, FileIds):
        json_json_body = json_body.to_dict()

    else:
        json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        Union[AnalysisStepFileData, AnalysisStepDatasetData, UnknownType],
        BadRequestError,
        ForbiddenError,
        NotFoundError,
    ]
]:
    if response.status_code == 200:

        def _parse_response_200(
            data: Union[Dict[str, Any]]
        ) -> Union[AnalysisStepFileData, AnalysisStepDatasetData, UnknownType]:
            response_200: Union[AnalysisStepFileData, AnalysisStepDatasetData, UnknownType]
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                analysis_step_data = AnalysisStepFileData.from_dict(data, strict=True)

                return analysis_step_data
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                analysis_step_data = AnalysisStepDatasetData.from_dict(data, strict=True)

                return analysis_step_data
            except:  # noqa: E722
                pass
            return UnknownType(data)

        response_200 = _parse_response_200(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = BadRequestError.from_dict(response.json(), strict=False)

        return response_400
    if response.status_code == 403:
        response_403 = ForbiddenError.from_dict(response.json(), strict=False)

        return response_403
    if response.status_code == 404:
        response_404 = NotFoundError.from_dict(response.json(), strict=False)

        return response_404
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        Union[AnalysisStepFileData, AnalysisStepDatasetData, UnknownType],
        BadRequestError,
        ForbiddenError,
        NotFoundError,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    analysis_step_data_id: str,
    json_body: Union[FileIds, AnalysisDatasetIds, UnknownType],
) -> Response[
    Union[
        Union[AnalysisStepFileData, AnalysisStepDatasetData, UnknownType],
        BadRequestError,
        ForbiddenError,
        NotFoundError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        analysis_step_data_id=analysis_step_data_id,
        json_body=json_body,
    )

    response = client.httpx_client.patch(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    analysis_step_data_id: str,
    json_body: Union[FileIds, AnalysisDatasetIds, UnknownType],
) -> Optional[
    Union[
        Union[AnalysisStepFileData, AnalysisStepDatasetData, UnknownType],
        BadRequestError,
        ForbiddenError,
        NotFoundError,
    ]
]:
    """Update a piece of data that is associated with a third-party analysis step.

    This endpoint accepts either file IDs or dataset IDs. If a step already has files or datasets attached
    to its step data, then these files or datasets will be overwritten when this endpoint is called. When
    this endpoint is called, the status of the analysis is updated to `WAITING`; the user should update
    the step status to `SUCCEEDED` immediately after the files or datasets have been successfully
    attached. The complete flow of attaching files or datasets to a step looks like the following:

    If attaching files:
    1. Upload the files with the [Create a file](#/Analyses/createFile) endpoint
    2. Use this endpoint to attach the files to a `FILE`-kind analysis step data
    3. Mark the analysis step as `SUCCEEDED` using the
       [Update an analysis step](#/Analyses/patchAnalysisStep) endpoint*

    If attaching datasets:
    1. Create the datasets from existing files using the [Create a dataset](#/Analyses/createDataset)
       endpoint
    2. Use this endpoint to attach the datasets to a `DATASET`-kind analysis step data
    3. Wait for all dataset upload statuses to change from `IN_PROGRESS` to `SUCCEEDED`, using the
       [Get a dataset](#/Analyses/getDataset) endpoint to check the status of each dataset
    3. Mark the analysis step as `SUCCEEDED` using the
       [Update an analysis step](#/Analyses/patchAnalysisStep) endpoint*

    *Note that in a future version of this API, the analysis step status will be automatically updated to
    `SUCCEEDED` after the files or datasets have been successfully attached, eliminating the need for this
    final endpoint call.
    """

    return sync_detailed(
        client=client,
        analysis_step_data_id=analysis_step_data_id,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    analysis_step_data_id: str,
    json_body: Union[FileIds, AnalysisDatasetIds, UnknownType],
) -> Response[
    Union[
        Union[AnalysisStepFileData, AnalysisStepDatasetData, UnknownType],
        BadRequestError,
        ForbiddenError,
        NotFoundError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        analysis_step_data_id=analysis_step_data_id,
        json_body=json_body,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.patch(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    analysis_step_data_id: str,
    json_body: Union[FileIds, AnalysisDatasetIds, UnknownType],
) -> Optional[
    Union[
        Union[AnalysisStepFileData, AnalysisStepDatasetData, UnknownType],
        BadRequestError,
        ForbiddenError,
        NotFoundError,
    ]
]:
    """Update a piece of data that is associated with a third-party analysis step.

    This endpoint accepts either file IDs or dataset IDs. If a step already has files or datasets attached
    to its step data, then these files or datasets will be overwritten when this endpoint is called. When
    this endpoint is called, the status of the analysis is updated to `WAITING`; the user should update
    the step status to `SUCCEEDED` immediately after the files or datasets have been successfully
    attached. The complete flow of attaching files or datasets to a step looks like the following:

    If attaching files:
    1. Upload the files with the [Create a file](#/Analyses/createFile) endpoint
    2. Use this endpoint to attach the files to a `FILE`-kind analysis step data
    3. Mark the analysis step as `SUCCEEDED` using the
       [Update an analysis step](#/Analyses/patchAnalysisStep) endpoint*

    If attaching datasets:
    1. Create the datasets from existing files using the [Create a dataset](#/Analyses/createDataset)
       endpoint
    2. Use this endpoint to attach the datasets to a `DATASET`-kind analysis step data
    3. Wait for all dataset upload statuses to change from `IN_PROGRESS` to `SUCCEEDED`, using the
       [Get a dataset](#/Analyses/getDataset) endpoint to check the status of each dataset
    3. Mark the analysis step as `SUCCEEDED` using the
       [Update an analysis step](#/Analyses/patchAnalysisStep) endpoint*

    *Note that in a future version of this API, the analysis step status will be automatically updated to
    `SUCCEEDED` after the files or datasets have been successfully attached, eliminating the need for this
    final endpoint call.
    """

    return (
        await asyncio_detailed(
            client=client,
            analysis_step_data_id=analysis_step_data_id,
            json_body=json_body,
        )
    ).parsed



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.workflow_task_schema import WorkflowTaskSchema
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowTaskSchemasPaginatedList")


@attr.s(auto_attribs=True, repr=False)
class WorkflowTaskSchemasPaginatedList:
    """  """

    _next_token: Union[Unset, str] = UNSET
    _workflow_task_schemas: Union[Unset, List[WorkflowTaskSchema]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("next_token={}".format(repr(self._next_token)))
        fields.append("workflow_task_schemas={}".format(repr(self._workflow_task_schemas)))
        return "WorkflowTaskSchemasPaginatedList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        next_token = self._next_token
        workflow_task_schemas: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._workflow_task_schemas, Unset):
            workflow_task_schemas = []
            for workflow_task_schemas_item_data in self._workflow_task_schemas:
                workflow_task_schemas_item = workflow_task_schemas_item_data.to_dict()

                workflow_task_schemas.append(workflow_task_schemas_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token
        if workflow_task_schemas is not UNSET:
            field_dict["workflowTaskSchemas"] = workflow_task_schemas

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        def get_workflow_task_schemas() -> Union[Unset, List[WorkflowTaskSchema]]:
            workflow_task_schemas = []
            _workflow_task_schemas = d.pop("workflowTaskSchemas")
            for workflow_task_schemas_item_data in _workflow_task_schemas or []:
                workflow_task_schemas_item = WorkflowTaskSchema.from_dict(
                    workflow_task_schemas_item_data, strict=False
                )

                workflow_task_schemas.append(workflow_task_schemas_item)

            return workflow_task_schemas

        try:
            workflow_task_schemas = get_workflow_task_schemas()
        except KeyError:
            if strict:
                raise
            workflow_task_schemas = cast(Union[Unset, List[WorkflowTaskSchema]], UNSET)

        workflow_task_schemas_paginated_list = cls(
            next_token=next_token,
            workflow_task_schemas=workflow_task_schemas,
        )

        return workflow_task_schemas_paginated_list

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET

    @property
    def workflow_task_schemas(self) -> List[WorkflowTaskSchema]:
        if isinstance(self._workflow_task_schemas, Unset):
            raise NotPresentError(self, "workflow_task_schemas")
        return self._workflow_task_schemas

    @workflow_task_schemas.setter
    def workflow_task_schemas(self, value: List[WorkflowTaskSchema]) -> None:
        self._workflow_task_schemas = value

    @workflow_task_schemas.deleter
    def workflow_task_schemas(self) -> None:
        self._workflow_task_schemas = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.request_task import RequestTask
from ..types import UNSET, Unset

T = TypeVar("T", bound="RequestTasksBulkUpdateResponse")


@attr.s(auto_attribs=True, repr=False)
class RequestTasksBulkUpdateResponse:
    """  """

    _tasks: Union[Unset, List[RequestTask]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("tasks={}".format(repr(self._tasks)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "RequestTasksBulkUpdateResponse({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        tasks: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._tasks, Unset):
            tasks = []
            for tasks_item_data in self._tasks:
                tasks_item = tasks_item_data.to_dict()

                tasks.append(tasks_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if tasks is not UNSET:
            field_dict["tasks"] = tasks

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_tasks() -> Union[Unset, List[RequestTask]]:
            tasks = []
            _tasks = d.pop("tasks")
            for tasks_item_data in _tasks or []:
                tasks_item = RequestTask.from_dict(tasks_item_data, strict=False)

                tasks.append(tasks_item)

            return tasks

        try:
            tasks = get_tasks()
        except KeyError:
            if strict:
                raise
            tasks = cast(Union[Unset, List[RequestTask]], UNSET)

        request_tasks_bulk_update_response = cls(
            tasks=tasks,
        )

        request_tasks_bulk_update_response.additional_properties = d
        return request_tasks_bulk_update_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def tasks(self) -> List[RequestTask]:
        """ The tasks to update """
        if isinstance(self._tasks, Unset):
            raise NotPresentError(self, "tasks")
        return self._tasks

    @tasks.setter
    def tasks(self, value: List[RequestTask]) -> None:
        self._tasks = value

    @tasks.deleter
    def tasks(self) -> None:
        self._tasks = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.free_form_text_form_field_canned_responses import FreeFormTextFormFieldCannedResponses
from ..models.free_form_text_form_field_default_keyboard import FreeFormTextFormFieldDefaultKeyboard
from ..models.free_form_text_form_field_type import FreeFormTextFormFieldType
from ..types import UNSET, Unset

T = TypeVar("T", bound="FreeFormTextFormField")


@attr.s(auto_attribs=True, repr=False)
class FreeFormTextFormField:
    """  """

    _canned_responses: Union[Unset, FreeFormTextFormFieldCannedResponses] = UNSET
    _default_keyboard: Union[Unset, FreeFormTextFormFieldDefaultKeyboard] = UNSET
    _number_of_lines: Union[Unset, float] = UNSET
    _type: Union[Unset, FreeFormTextFormFieldType] = UNSET
    _description: Union[Unset, str] = UNSET
    _is_required: Union[Unset, bool] = UNSET
    _key: Union[Unset, str] = UNSET
    _label: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("canned_responses={}".format(repr(self._canned_responses)))
        fields.append("default_keyboard={}".format(repr(self._default_keyboard)))
        fields.append("number_of_lines={}".format(repr(self._number_of_lines)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("description={}".format(repr(self._description)))
        fields.append("is_required={}".format(repr(self._is_required)))
        fields.append("key={}".format(repr(self._key)))
        fields.append("label={}".format(repr(self._label)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "FreeFormTextFormField({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        canned_responses: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._canned_responses, Unset):
            canned_responses = self._canned_responses.to_dict()

        default_keyboard: Union[Unset, int] = UNSET
        if not isinstance(self._default_keyboard, Unset):
            default_keyboard = self._default_keyboard.value

        number_of_lines = self._number_of_lines
        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        description = self._description
        is_required = self._is_required
        key = self._key
        label = self._label

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if canned_responses is not UNSET:
            field_dict["cannedResponses"] = canned_responses
        if default_keyboard is not UNSET:
            field_dict["defaultKeyboard"] = default_keyboard
        if number_of_lines is not UNSET:
            field_dict["numberOfLines"] = number_of_lines
        if type is not UNSET:
            field_dict["type"] = type
        if description is not UNSET:
            field_dict["description"] = description
        if is_required is not UNSET:
            field_dict["isRequired"] = is_required
        if key is not UNSET:
            field_dict["key"] = key
        if label is not UNSET:
            field_dict["label"] = label

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_canned_responses() -> Union[Unset, FreeFormTextFormFieldCannedResponses]:
            canned_responses: Union[Unset, Union[Unset, FreeFormTextFormFieldCannedResponses]] = UNSET
            _canned_responses = d.pop("cannedResponses")

            if not isinstance(_canned_responses, Unset):
                canned_responses = FreeFormTextFormFieldCannedResponses.from_dict(_canned_responses)

            return canned_responses

        try:
            canned_responses = get_canned_responses()
        except KeyError:
            if strict:
                raise
            canned_responses = cast(Union[Unset, FreeFormTextFormFieldCannedResponses], UNSET)

        def get_default_keyboard() -> Union[Unset, FreeFormTextFormFieldDefaultKeyboard]:
            default_keyboard = UNSET
            _default_keyboard = d.pop("defaultKeyboard")
            if _default_keyboard is not None and _default_keyboard is not UNSET:
                try:
                    default_keyboard = FreeFormTextFormFieldDefaultKeyboard(_default_keyboard)
                except ValueError:
                    default_keyboard = FreeFormTextFormFieldDefaultKeyboard.of_unknown(_default_keyboard)

            return default_keyboard

        try:
            default_keyboard = get_default_keyboard()
        except KeyError:
            if strict:
                raise
            default_keyboard = cast(Union[Unset, FreeFormTextFormFieldDefaultKeyboard], UNSET)

        def get_number_of_lines() -> Union[Unset, float]:
            number_of_lines = d.pop("numberOfLines")
            return number_of_lines

        try:
            number_of_lines = get_number_of_lines()
        except KeyError:
            if strict:
                raise
            number_of_lines = cast(Union[Unset, float], UNSET)

        def get_type() -> Union[Unset, FreeFormTextFormFieldType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = FreeFormTextFormFieldType(_type)
                except ValueError:
                    type = FreeFormTextFormFieldType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, FreeFormTextFormFieldType], UNSET)

        def get_description() -> Union[Unset, str]:
            description = d.pop("description")
            return description

        try:
            description = get_description()
        except KeyError:
            if strict:
                raise
            description = cast(Union[Unset, str], UNSET)

        def get_is_required() -> Union[Unset, bool]:
            is_required = d.pop("isRequired")
            return is_required

        try:
            is_required = get_is_required()
        except KeyError:
            if strict:
                raise
            is_required = cast(Union[Unset, bool], UNSET)

        def get_key() -> Union[Unset, str]:
            key = d.pop("key")
            return key

        try:
            key = get_key()
        except KeyError:
            if strict:
                raise
            key = cast(Union[Unset, str], UNSET)

        def get_label() -> Union[Unset, str]:
            label = d.pop("label")
            return label

        try:
            label = get_label()
        except KeyError:
            if strict:
                raise
            label = cast(Union[Unset, str], UNSET)

        free_form_text_form_field = cls(
            canned_responses=canned_responses,
            default_keyboard=default_keyboard,
            number_of_lines=number_of_lines,
            type=type,
            description=description,
            is_required=is_required,
            key=key,
            label=label,
        )

        free_form_text_form_field.additional_properties = d
        return free_form_text_form_field

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def canned_responses(self) -> FreeFormTextFormFieldCannedResponses:
        if isinstance(self._canned_responses, Unset):
            raise NotPresentError(self, "canned_responses")
        return self._canned_responses

    @canned_responses.setter
    def canned_responses(self, value: FreeFormTextFormFieldCannedResponses) -> None:
        self._canned_responses = value

    @canned_responses.deleter
    def canned_responses(self) -> None:
        self._canned_responses = UNSET

    @property
    def default_keyboard(self) -> FreeFormTextFormFieldDefaultKeyboard:
        if isinstance(self._default_keyboard, Unset):
            raise NotPresentError(self, "default_keyboard")
        return self._default_keyboard

    @default_keyboard.setter
    def default_keyboard(self, value: FreeFormTextFormFieldDefaultKeyboard) -> None:
        self._default_keyboard = value

    @default_keyboard.deleter
    def default_keyboard(self) -> None:
        self._default_keyboard = UNSET

    @property
    def number_of_lines(self) -> float:
        if isinstance(self._number_of_lines, Unset):
            raise NotPresentError(self, "number_of_lines")
        return self._number_of_lines

    @number_of_lines.setter
    def number_of_lines(self, value: float) -> None:
        self._number_of_lines = value

    @number_of_lines.deleter
    def number_of_lines(self) -> None:
        self._number_of_lines = UNSET

    @property
    def type(self) -> FreeFormTextFormFieldType:
        """The type of this form field. Type declares how this field behaves and dictates the additional properties passed along with the required properties like label and key"""
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: FreeFormTextFormFieldType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET

    @property
    def description(self) -> str:
        """ Description of the purpose of this field """
        if isinstance(self._description, Unset):
            raise NotPresentError(self, "description")
        return self._description

    @description.setter
    def description(self, value: str) -> None:
        self._description = value

    @description.deleter
    def description(self) -> None:
        self._description = UNSET

    @property
    def is_required(self) -> bool:
        """ Whether this field is required to be filled out in order to be a valid submission """
        if isinstance(self._is_required, Unset):
            raise NotPresentError(self, "is_required")
        return self._is_required

    @is_required.setter
    def is_required(self, value: bool) -> None:
        self._is_required = value

    @is_required.deleter
    def is_required(self) -> None:
        self._is_required = UNSET

    @property
    def key(self) -> str:
        """ Reference key of this form field. Used to fix identity of fields beyond the label """
        if isinstance(self._key, Unset):
            raise NotPresentError(self, "key")
        return self._key

    @key.setter
    def key(self, value: str) -> None:
        self._key = value

    @key.deleter
    def key(self) -> None:
        self._key = UNSET

    @property
    def label(self) -> str:
        """ End user facing name of this form field. What you see when you fill out the form each time """
        if isinstance(self._label, Unset):
            raise NotPresentError(self, "label")
        return self._label

    @label.setter
    def label(self, value: str) -> None:
        self._label = value

    @label.deleter
    def label(self) -> None:
        self._label = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.form_definition import FormDefinition
from ..types import UNSET, Unset

T = TypeVar("T", bound="FormDefinitionsPaginatedList")


@attr.s(auto_attribs=True, repr=False)
class FormDefinitionsPaginatedList:
    """  """

    _form_definitions: Union[Unset, List[FormDefinition]] = UNSET
    _next_token: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("form_definitions={}".format(repr(self._form_definitions)))
        fields.append("next_token={}".format(repr(self._next_token)))
        return "FormDefinitionsPaginatedList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        form_definitions: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._form_definitions, Unset):
            form_definitions = []
            for form_definitions_item_data in self._form_definitions:
                form_definitions_item = form_definitions_item_data.to_dict()

                form_definitions.append(form_definitions_item)

        next_token = self._next_token

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if form_definitions is not UNSET:
            field_dict["formDefinitions"] = form_definitions
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_form_definitions() -> Union[Unset, List[FormDefinition]]:
            form_definitions = []
            _form_definitions = d.pop("formDefinitions")
            for form_definitions_item_data in _form_definitions or []:
                form_definitions_item = FormDefinition.from_dict(form_definitions_item_data, strict=False)

                form_definitions.append(form_definitions_item)

            return form_definitions

        try:
            form_definitions = get_form_definitions()
        except KeyError:
            if strict:
                raise
            form_definitions = cast(Union[Unset, List[FormDefinition]], UNSET)

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        form_definitions_paginated_list = cls(
            form_definitions=form_definitions,
            next_token=next_token,
        )

        return form_definitions_paginated_list

    @property
    def form_definitions(self) -> List[FormDefinition]:
        if isinstance(self._form_definitions, Unset):
            raise NotPresentError(self, "form_definitions")
        return self._form_definitions

    @form_definitions.setter
    def form_definitions(self, value: List[FormDefinition]) -> None:
        self._form_definitions = value

    @form_definitions.deleter
    def form_definitions(self) -> None:
        self._form_definitions = UNSET

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.async_task_link import AsyncTaskLink
from ...models.bad_request_error import BadRequestError
from ...models.custom_entities_bulk_create_request import CustomEntitiesBulkCreateRequest
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    json_body: CustomEntitiesBulkCreateRequest,
) -> Dict[str, Any]:
    url = "{}/custom-entities:bulk-create".format(client.base_url)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    if response.status_code == 202:
        response_202 = AsyncTaskLink.from_dict(response.json(), strict=False)

        return response_202
    if response.status_code == 400:
        response_400 = BadRequestError.from_dict(response.json(), strict=False)

        return response_400
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: CustomEntitiesBulkCreateRequest,
) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = client.httpx_client.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    json_body: CustomEntitiesBulkCreateRequest,
) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    """ Bulk Create custom entities. Limit of 2500 custom entities per request. """

    return sync_detailed(
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    json_body: CustomEntitiesBulkCreateRequest,
) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    json_body: CustomEntitiesBulkCreateRequest,
) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    """ Bulk Create custom entities. Limit of 2500 custom entities per request. """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
        )
    ).parsed



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="DnaSequencesFindMatchingRegion")


@attr.s(auto_attribs=True, repr=False)
class DnaSequencesFindMatchingRegion:
    """  """

    _schema_id: str
    _target_dna_sequence_ids: List[str]
    _registry_id: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("schema_id={}".format(repr(self._schema_id)))
        fields.append("target_dna_sequence_ids={}".format(repr(self._target_dna_sequence_ids)))
        fields.append("registry_id={}".format(repr(self._registry_id)))
        return "DnaSequencesFindMatchingRegion({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        schema_id = self._schema_id
        target_dna_sequence_ids = self._target_dna_sequence_ids

        registry_id = self._registry_id

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if schema_id is not UNSET:
            field_dict["schemaId"] = schema_id
        if target_dna_sequence_ids is not UNSET:
            field_dict["targetDnaSequenceIds"] = target_dna_sequence_ids
        if registry_id is not UNSET:
            field_dict["registryId"] = registry_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_schema_id() -> str:
            schema_id = d.pop("schemaId")
            return schema_id

        try:
            schema_id = get_schema_id()
        except KeyError:
            if strict:
                raise
            schema_id = cast(str, UNSET)

        def get_target_dna_sequence_ids() -> List[str]:
            target_dna_sequence_ids = cast(List[str], d.pop("targetDnaSequenceIds"))

            return target_dna_sequence_ids

        try:
            target_dna_sequence_ids = get_target_dna_sequence_ids()
        except KeyError:
            if strict:
                raise
            target_dna_sequence_ids = cast(List[str], UNSET)

        def get_registry_id() -> Union[Unset, str]:
            registry_id = d.pop("registryId")
            return registry_id

        try:
            registry_id = get_registry_id()
        except KeyError:
            if strict:
                raise
            registry_id = cast(Union[Unset, str], UNSET)

        dna_sequences_find_matching_region = cls(
            schema_id=schema_id,
            target_dna_sequence_ids=target_dna_sequence_ids,
            registry_id=registry_id,
        )

        return dna_sequences_find_matching_region

    @property
    def schema_id(self) -> str:
        """ API ID for an Entity Schema. Restricts results to DNA Sequences of this schema type. """
        if isinstance(self._schema_id, Unset):
            raise NotPresentError(self, "schema_id")
        return self._schema_id

    @schema_id.setter
    def schema_id(self, value: str) -> None:
        self._schema_id = value

    @property
    def target_dna_sequence_ids(self) -> List[str]:
        """ API IDs of the DNA sequences which matching regions will be found for """
        if isinstance(self._target_dna_sequence_ids, Unset):
            raise NotPresentError(self, "target_dna_sequence_ids")
        return self._target_dna_sequence_ids

    @target_dna_sequence_ids.setter
    def target_dna_sequence_ids(self, value: List[str]) -> None:
        self._target_dna_sequence_ids = value

    @property
    def registry_id(self) -> str:
        """ An optional Registry ID to restrict the region search to """
        if isinstance(self._registry_id, Unset):
            raise NotPresentError(self, "registry_id")
        return self._registry_id

    @registry_id.setter
    def registry_id(self, value: str) -> None:
        self._registry_id = value

    @registry_id.deleter
    def registry_id(self) -> None:
        self._registry_id = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="PrintLabels")


@attr.s(auto_attribs=True, repr=False)
class PrintLabels:
    """  """

    _container_ids: List[str]
    _label_template_id: str
    _printer_id: str

    def __repr__(self):
        fields = []
        fields.append("container_ids={}".format(repr(self._container_ids)))
        fields.append("label_template_id={}".format(repr(self._label_template_id)))
        fields.append("printer_id={}".format(repr(self._printer_id)))
        return "PrintLabels({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        container_ids = self._container_ids

        label_template_id = self._label_template_id
        printer_id = self._printer_id

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if container_ids is not UNSET:
            field_dict["containerIds"] = container_ids
        if label_template_id is not UNSET:
            field_dict["labelTemplateId"] = label_template_id
        if printer_id is not UNSET:
            field_dict["printerId"] = printer_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_container_ids() -> List[str]:
            container_ids = cast(List[str], d.pop("containerIds"))

            return container_ids

        try:
            container_ids = get_container_ids()
        except KeyError:
            if strict:
                raise
            container_ids = cast(List[str], UNSET)

        def get_label_template_id() -> str:
            label_template_id = d.pop("labelTemplateId")
            return label_template_id

        try:
            label_template_id = get_label_template_id()
        except KeyError:
            if strict:
                raise
            label_template_id = cast(str, UNSET)

        def get_printer_id() -> str:
            printer_id = d.pop("printerId")
            return printer_id

        try:
            printer_id = get_printer_id()
        except KeyError:
            if strict:
                raise
            printer_id = cast(str, UNSET)

        print_labels = cls(
            container_ids=container_ids,
            label_template_id=label_template_id,
            printer_id=printer_id,
        )

        return print_labels

    @property
    def container_ids(self) -> List[str]:
        """List of IDs of containers that will have labels printed (one label will be printed per container)."""
        if isinstance(self._container_ids, Unset):
            raise NotPresentError(self, "container_ids")
        return self._container_ids

    @container_ids.setter
    def container_ids(self, value: List[str]) -> None:
        self._container_ids = value

    @property
    def label_template_id(self) -> str:
        """ID of label template to use (same template will be used for all labels printed)."""
        if isinstance(self._label_template_id, Unset):
            raise NotPresentError(self, "label_template_id")
        return self._label_template_id

    @label_template_id.setter
    def label_template_id(self, value: str) -> None:
        self._label_template_id = value

    @property
    def printer_id(self) -> str:
        """ID of printer to use (same printer will be used for all labels printed)."""
        if isinstance(self._printer_id, Unset):
            raise NotPresentError(self, "printer_id")
        return self._printer_id

    @printer_id.setter
    def printer_id(self, value: str) -> None:
        self._printer_id = value



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.fields import Fields
from ..models.sample_restriction_status import SampleRestrictionStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="ContainerWriteBase")


@attr.s(auto_attribs=True, repr=False)
class ContainerWriteBase:
    """  """

    _fields: Union[Unset, Fields] = UNSET
    _name: Union[Unset, str] = UNSET
    _parent_storage_id: Union[Unset, str] = UNSET
    _restricted_sample_party_ids: Union[Unset, List[str]] = UNSET
    _restriction_status: Union[Unset, SampleRestrictionStatus] = UNSET
    _sample_owner_ids: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("fields={}".format(repr(self._fields)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("parent_storage_id={}".format(repr(self._parent_storage_id)))
        fields.append("restricted_sample_party_ids={}".format(repr(self._restricted_sample_party_ids)))
        fields.append("restriction_status={}".format(repr(self._restriction_status)))
        fields.append("sample_owner_ids={}".format(repr(self._sample_owner_ids)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "ContainerWriteBase({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._fields, Unset):
            fields = self._fields.to_dict()

        name = self._name
        parent_storage_id = self._parent_storage_id
        restricted_sample_party_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._restricted_sample_party_ids, Unset):
            restricted_sample_party_ids = self._restricted_sample_party_ids

        restriction_status: Union[Unset, int] = UNSET
        if not isinstance(self._restriction_status, Unset):
            restriction_status = self._restriction_status.value

        sample_owner_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._sample_owner_ids, Unset):
            sample_owner_ids = self._sample_owner_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if fields is not UNSET:
            field_dict["fields"] = fields
        if name is not UNSET:
            field_dict["name"] = name
        if parent_storage_id is not UNSET:
            field_dict["parentStorageId"] = parent_storage_id
        if restricted_sample_party_ids is not UNSET:
            field_dict["restrictedSamplePartyIds"] = restricted_sample_party_ids
        if restriction_status is not UNSET:
            field_dict["restrictionStatus"] = restriction_status
        if sample_owner_ids is not UNSET:
            field_dict["sampleOwnerIds"] = sample_owner_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_fields() -> Union[Unset, Fields]:
            fields: Union[Unset, Union[Unset, Fields]] = UNSET
            _fields = d.pop("fields")

            if not isinstance(_fields, Unset):
                fields = Fields.from_dict(_fields)

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Unset, Fields], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_parent_storage_id() -> Union[Unset, str]:
            parent_storage_id = d.pop("parentStorageId")
            return parent_storage_id

        try:
            parent_storage_id = get_parent_storage_id()
        except KeyError:
            if strict:
                raise
            parent_storage_id = cast(Union[Unset, str], UNSET)

        def get_restricted_sample_party_ids() -> Union[Unset, List[str]]:
            restricted_sample_party_ids = cast(List[str], d.pop("restrictedSamplePartyIds"))

            return restricted_sample_party_ids

        try:
            restricted_sample_party_ids = get_restricted_sample_party_ids()
        except KeyError:
            if strict:
                raise
            restricted_sample_party_ids = cast(Union[Unset, List[str]], UNSET)

        def get_restriction_status() -> Union[Unset, SampleRestrictionStatus]:
            restriction_status = UNSET
            _restriction_status = d.pop("restrictionStatus")
            if _restriction_status is not None and _restriction_status is not UNSET:
                try:
                    restriction_status = SampleRestrictionStatus(_restriction_status)
                except ValueError:
                    restriction_status = SampleRestrictionStatus.of_unknown(_restriction_status)

            return restriction_status

        try:
            restriction_status = get_restriction_status()
        except KeyError:
            if strict:
                raise
            restriction_status = cast(Union[Unset, SampleRestrictionStatus], UNSET)

        def get_sample_owner_ids() -> Union[Unset, List[str]]:
            sample_owner_ids = cast(List[str], d.pop("sampleOwnerIds"))

            return sample_owner_ids

        try:
            sample_owner_ids = get_sample_owner_ids()
        except KeyError:
            if strict:
                raise
            sample_owner_ids = cast(Union[Unset, List[str]], UNSET)

        container_write_base = cls(
            fields=fields,
            name=name,
            parent_storage_id=parent_storage_id,
            restricted_sample_party_ids=restricted_sample_party_ids,
            restriction_status=restriction_status,
            sample_owner_ids=sample_owner_ids,
        )

        container_write_base.additional_properties = d
        return container_write_base

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def fields(self) -> Fields:
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Fields) -> None:
        self._fields = value

    @fields.deleter
    def fields(self) -> None:
        self._fields = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def parent_storage_id(self) -> str:
        """ ID of containing parent inventory, can also specify a coordinate for plates and boxes (e.g. plt_2bAks9dx:a2). """
        if isinstance(self._parent_storage_id, Unset):
            raise NotPresentError(self, "parent_storage_id")
        return self._parent_storage_id

    @parent_storage_id.setter
    def parent_storage_id(self, value: str) -> None:
        self._parent_storage_id = value

    @parent_storage_id.deleter
    def parent_storage_id(self) -> None:
        self._parent_storage_id = UNSET

    @property
    def restricted_sample_party_ids(self) -> List[str]:
        """IDs of users or teams who have access to use a restricted container. Fixed plate wells and unrestricted containers do not have restricted sample parties."""
        if isinstance(self._restricted_sample_party_ids, Unset):
            raise NotPresentError(self, "restricted_sample_party_ids")
        return self._restricted_sample_party_ids

    @restricted_sample_party_ids.setter
    def restricted_sample_party_ids(self, value: List[str]) -> None:
        self._restricted_sample_party_ids = value

    @restricted_sample_party_ids.deleter
    def restricted_sample_party_ids(self) -> None:
        self._restricted_sample_party_ids = UNSET

    @property
    def restriction_status(self) -> SampleRestrictionStatus:
        if isinstance(self._restriction_status, Unset):
            raise NotPresentError(self, "restriction_status")
        return self._restriction_status

    @restriction_status.setter
    def restriction_status(self, value: SampleRestrictionStatus) -> None:
        self._restriction_status = value

    @restriction_status.deleter
    def restriction_status(self) -> None:
        self._restriction_status = UNSET

    @property
    def sample_owner_ids(self) -> List[str]:
        """IDs of users or teams who are sample owners for the container. Fixed plate wells do not have sample owners."""
        if isinstance(self._sample_owner_ids, Unset):
            raise NotPresentError(self, "sample_owner_ids")
        return self._sample_owner_ids

    @sample_owner_ids.setter
    def sample_owner_ids(self, value: List[str]) -> None:
        self._sample_owner_ids = value

    @sample_owner_ids.deleter
    def sample_owner_ids(self) -> None:
        self._sample_owner_ids = UNSET



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="InitialTable")


@attr.s(auto_attribs=True, repr=False)
class InitialTable:
    """  """

    _csv_data: Union[Unset, str] = UNSET
    _template_table_id: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("csv_data={}".format(repr(self._csv_data)))
        fields.append("template_table_id={}".format(repr(self._template_table_id)))
        return "InitialTable({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        csv_data = self._csv_data
        template_table_id = self._template_table_id

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if csv_data is not UNSET:
            field_dict["csvData"] = csv_data
        if template_table_id is not UNSET:
            field_dict["templateTableID"] = template_table_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_csv_data() -> Union[Unset, str]:
            csv_data = d.pop("csvData")
            return csv_data

        try:
            csv_data = get_csv_data()
        except KeyError:
            if strict:
                raise
            csv_data = cast(Union[Unset, str], UNSET)

        def get_template_table_id() -> Union[Unset, str]:
            template_table_id = d.pop("templateTableID")
            return template_table_id

        try:
            template_table_id = get_template_table_id()
        except KeyError:
            if strict:
                raise
            template_table_id = cast(Union[Unset, str], UNSET)

        initial_table = cls(
            csv_data=csv_data,
            template_table_id=template_table_id,
        )

        return initial_table

    @property
    def csv_data(self) -> str:
        """ blobId of an uploaded csv blob. The CSV should be formatted with column headers of `columnId` which can be found in the [EntryTemplate](#/components/schemas/EntryTemplate). For more information on uploading a blob, [click here](https://docs.benchling.com/docs/uploading-a-blob-to-benchling). """
        if isinstance(self._csv_data, Unset):
            raise NotPresentError(self, "csv_data")
        return self._csv_data

    @csv_data.setter
    def csv_data(self, value: str) -> None:
        self._csv_data = value

    @csv_data.deleter
    def csv_data(self) -> None:
        self._csv_data = UNSET

    @property
    def template_table_id(self) -> str:
        """ Template table API ID """
        if isinstance(self._template_table_id, Unset):
            raise NotPresentError(self, "template_table_id")
        return self._template_table_id

    @template_table_id.setter
    def template_table_id(self, value: str) -> None:
        self._template_table_id = value

    @template_table_id.deleter
    def template_table_id(self) -> None:
        self._template_table_id = UNSET



from typing import Union

from ..extensions import UnknownType
from ..models.barcode_scan_form_instance_provider import BarcodeScanFormInstanceProvider
from ..models.entity_search_instance_provider import EntitySearchInstanceProvider
from ..models.new_blank_form_instance_provider import NewBlankFormInstanceProvider

InstanceProvider = Union[
    BarcodeScanFormInstanceProvider, NewBlankFormInstanceProvider, EntitySearchInstanceProvider, UnknownType
]



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="AutoAnnotateAaSequences")


@attr.s(auto_attribs=True, repr=False)
class AutoAnnotateAaSequences:
    """  """

    _aa_sequence_ids: List[str]
    _feature_library_ids: List[str]

    def __repr__(self):
        fields = []
        fields.append("aa_sequence_ids={}".format(repr(self._aa_sequence_ids)))
        fields.append("feature_library_ids={}".format(repr(self._feature_library_ids)))
        return "AutoAnnotateAaSequences({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        aa_sequence_ids = self._aa_sequence_ids

        feature_library_ids = self._feature_library_ids

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if aa_sequence_ids is not UNSET:
            field_dict["aaSequenceIds"] = aa_sequence_ids
        if feature_library_ids is not UNSET:
            field_dict["featureLibraryIds"] = feature_library_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_aa_sequence_ids() -> List[str]:
            aa_sequence_ids = cast(List[str], d.pop("aaSequenceIds"))

            return aa_sequence_ids

        try:
            aa_sequence_ids = get_aa_sequence_ids()
        except KeyError:
            if strict:
                raise
            aa_sequence_ids = cast(List[str], UNSET)

        def get_feature_library_ids() -> List[str]:
            feature_library_ids = cast(List[str], d.pop("featureLibraryIds"))

            return feature_library_ids

        try:
            feature_library_ids = get_feature_library_ids()
        except KeyError:
            if strict:
                raise
            feature_library_ids = cast(List[str], UNSET)

        auto_annotate_aa_sequences = cls(
            aa_sequence_ids=aa_sequence_ids,
            feature_library_ids=feature_library_ids,
        )

        return auto_annotate_aa_sequences

    @property
    def aa_sequence_ids(self) -> List[str]:
        """ Array of AA sequence IDs. """
        if isinstance(self._aa_sequence_ids, Unset):
            raise NotPresentError(self, "aa_sequence_ids")
        return self._aa_sequence_ids

    @aa_sequence_ids.setter
    def aa_sequence_ids(self, value: List[str]) -> None:
        self._aa_sequence_ids = value

    @property
    def feature_library_ids(self) -> List[str]:
        """ Array of feature library IDs. """
        if isinstance(self._feature_library_ids, Unset):
            raise NotPresentError(self, "feature_library_ids")
        return self._feature_library_ids

    @feature_library_ids.setter
    def feature_library_ids(self, value: List[str]) -> None:
        self._feature_library_ids = value



from typing import Any, cast, Dict, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="AutomationInputGeneratorUpdate")


@attr.s(auto_attribs=True, repr=False)
class AutomationInputGeneratorUpdate:
    """  """

    _file_id: Union[Unset, None, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("file_id={}".format(repr(self._file_id)))
        return "AutomationInputGeneratorUpdate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        file_id = self._file_id

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if file_id is not UNSET:
            field_dict["fileId"] = file_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_file_id() -> Union[Unset, None, str]:
            file_id = d.pop("fileId")
            return file_id

        try:
            file_id = get_file_id()
        except KeyError:
            if strict:
                raise
            file_id = cast(Union[Unset, None, str], UNSET)

        automation_input_generator_update = cls(
            file_id=file_id,
        )

        return automation_input_generator_update

    @property
    def file_id(self) -> Optional[str]:
        """ The ID of the file (blob) associated with the input generator. Set to `null` to remove an existing file from the generator. """
        if isinstance(self._file_id, Unset):
            raise NotPresentError(self, "file_id")
        return self._file_id

    @file_id.setter
    def file_id(self, value: Optional[str]) -> None:
        self._file_id = value

    @file_id.deleter
    def file_id(self) -> None:
        self._file_id = UNSET
